@@ -26,14 +26,15 @@ namespace rfb {
   public:
     CopyRectDecoder();
     virtual ~CopyRectDecoder();
-    virtual bool readRect(const Rect& r, rdr::InStream* is,
-                          const ServerParams& server, rdr::OutStream* os);
-    virtual void getAffectedRegion(const Rect& rect, const uint8_t* buffer,
-                                   size_t buflen, const ServerParams& server,
-                                   Region* region);
-    virtual void decodeRect(const Rect& r, const uint8_t* buffer,
-                            size_t buflen, const ServerParams& server,
-                            ModifiablePixelBuffer* pb);
+    bool readRect(const core::Rect& r, rdr::InStream* is,
+                  const ServerParams& server,
+                  rdr::OutStream* os) override;
+    void getAffectedRegion(const core::Rect& rect, const uint8_t* buffer,
+                           size_t buflen, const ServerParams& server,
+                           core::Region* region) override;
+    void decodeRect(const core::Rect& r, const uint8_t* buffer,
+                    size_t buflen, const ServerParams& server,
+                    ModifiablePixelBuffer* pb) override;
   };
 }
 #endif
