@@ -16,20 +16,31 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  * USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
+
+#include <core/Region.h>
+
 #include <rfb/encodings.h>
-#include <rfb/Region.h>
 #include <rfb/Decoder.h>
 #include <rfb/RawDecoder.h>
 #include <rfb/CopyRectDecoder.h>
 #include <rfb/RREDecoder.h>
 #include <rfb/HextileDecoder.h>
+#include <rfb/JPEGDecoder.h>
 #include <rfb/ZRLEDecoder.h>
 #include <rfb/TightDecoder.h>
+#ifdef HAVE_H264
+#include <rfb/H264Decoder.h>
+#endif
 
 using namespace rfb;
 
-Decoder::Decoder(enum DecoderFlags flags) : flags(flags)
+Decoder::Decoder(enum DecoderFlags flags_) : flags(flags_)
 {
 }
 
@@ -37,17 +48,22 @@ Decoder::~Decoder()
 {
 }
 
-void Decoder::getAffectedRegion(const Rect& rect, const void* buffer,
-                                size_t buflen, const ConnParams& cp,
-                                Region* region)
+void Decoder::getAffectedRegion(const core::Rect& rect,
+                                const uint8_t* /*buffer*/,
+                                size_t /*buflen*/,
+                                const ServerParams& /*server*/,
+                                core::Region* region)
 {
   region->reset(rect);
 }
 
-bool Decoder::doRectsConflict(const Rect& rectA, const void* bufferA,
-                              size_t buflenA, const Rect& rectB,
-                              const void* bufferB, size_t buflenB,
-                              const ConnParams& cp)
+bool Decoder::doRectsConflict(const core::Rect& /*rectA*/,
+                              const uint8_t* /*bufferA*/,
+                              size_t /*buflenA*/,
+                              const core::Rect& /*rectB*/,
+                              const uint8_t* /*bufferB*/,
+                              size_t /*buflenB*/,
+                              const ServerParams& /*server*/)
 {
   return false;
 }
@@ -59,8 +75,12 @@ bool Decoder::supported(int encoding)
   case encodingCopyRect:
   case encodingRRE:
   case encodingHextile:
+  case encodingJPEG:
   case encodingZRLE:
   case encodingTight:
+#ifdef HAVE_H264
+  case encodingH264:
+#endif
     return true;
   default:
     return false;
@@ -78,11 +98,17 @@ Decoder* Decoder::createDecoder(int encoding)
     return new RREDecoder();
   case encodingHextile:
     return new HextileDecoder();
+  case encodingJPEG:
+    return new JPEGDecoder();
   case encodingZRLE:
     return new ZRLEDecoder();
   case encodingTight:
     return new TightDecoder();
+#ifdef HAVE_H264
+  case encodingH264:
+    return new H264Decoder();
+#endif
   default:
-    return NULL;
+    return nullptr;
   }
 }
