@@ -1,6 +1,6 @@
 /* Copyright (C) 2000-2003 Constantin Kaplinsky.  All Rights Reserved.
  * Copyright (C) 2011 D. R. Commander.  All Rights Reserved.
- * Copyright 2014-2018 Pierre Ossman for Cendio AB
+ * Copyright 2014-2025 Pierre Ossman for Cendio AB
  * Copyright 2018 Peter Astrand for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
@@ -19,26 +19,35 @@
  * USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 
+#include <core/LogWriter.h>
+#include <core/string.h>
+
+#include <rfb/Cursor.h>
 #include <rfb/EncodeManager.h>
 #include <rfb/Encoder.h>
 #include <rfb/Palette.h>
 #include <rfb/SConnection.h>
 #include <rfb/SMsgWriter.h>
 #include <rfb/UpdateTracker.h>
-#include <rfb/LogWriter.h>
+#include <rfb/encodings.h>
 
 #include <rfb/RawEncoder.h>
 #include <rfb/RREEncoder.h>
 #include <rfb/HextileEncoder.h>
+#include <rfb/JPEGEncoder.h>
 #include <rfb/ZRLEEncoder.h>
 #include <rfb/TightEncoder.h>
 #include <rfb/TightJPEGEncoder.h>
 
 using namespace rfb;
 
-static LogWriter vlog("EncodeManager");
+static core::LogWriter vlog("EncodeManager");
 
 // Split each rectangle into smaller ones no larger than this area,
 // and no wider than this width.
@@ -63,6 +72,7 @@ enum EncoderClass {
   encoderTight,
   encoderTightJPEG,
   encoderZRLE,
+  encoderJPEG,
   encoderClassMax,
 };
 
@@ -98,6 +108,8 @@ static const char *encoderClassName(EncoderClass klass)
     return "Tight (JPEG)";
   case encoderZRLE:
     return "ZRLE";
+  case encoderJPEG:
+    return "JPEG";
   case encoderClassMax:
     break;
   }
@@ -132,7 +144,7 @@ EncodeManager::EncodeManager(SConnection* conn_)
 {
   StatsVector::iterator iter;
 
-  encoders.resize(encoderClassMax, NULL);
+  encoders.resize(encoderClassMax, nullptr);
   activeEncoders.resize(encoderTypeMax, encoderRaw);
 
   encoders[encoderRaw] = new RawEncoder(conn);
@@ -141,6 +153,7 @@ EncodeManager::EncodeManager(SConnection* conn_)
   encoders[encoderTight] = new TightEncoder(conn);
   encoders[encoderTightJPEG] = new TightJPEGEncoder(conn);
   encoders[encoderZRLE] = new ZRLEEncoder(conn);
+  encoders[encoderJPEG] = new JPEGEncoder(conn);
 
   updates = 0;
   memset(&copyStats, 0, sizeof(copyStats));
@@ -155,12 +168,10 @@ EncodeManager::EncodeManager(SConnection* conn_)
 
 EncodeManager::~EncodeManager()
 {
-  std::vector<Encoder*>::iterator iter;
-
   logStats();
 
-  for (iter = encoders.begin();iter != encoders.end();iter++)
-    delete *iter;
+  for (Encoder* encoder : encoders)
+    delete encoder;
 }
 
 void EncodeManager::logStats()
@@ -172,8 +183,6 @@ void EncodeManager::logStats()
 
   double ratio;
 
-  char a[1024], b[1024];
-
   rects = 0;
   pixels = bytes = equivalent = 0;
 
@@ -189,13 +198,12 @@ void EncodeManager::logStats()
 
     ratio = (double)copyStats.equivalent / copyStats.bytes;
 
-    siPrefix(copyStats.rects, "rects", a, sizeof(a));
-    siPrefix(copyStats.pixels, "pixels", b, sizeof(b));
-    vlog.info("    %s: %s, %s", "Copies", a, b);
-    iecPrefix(copyStats.bytes, "B", a, sizeof(a));
+    vlog.info("    %s: %s, %s", "Copies",
+              core::siPrefix(copyStats.rects, "rects").c_str(),
+              core::siPrefix(copyStats.pixels, "pixels").c_str());
     vlog.info("    %*s  %s (1:%g ratio)",
               (int)strlen("Copies"), "",
-              a, ratio);
+              core::iecPrefix(copyStats.bytes, "B").c_str(), ratio);
   }
 
   for (i = 0;i < stats.size();i++) {
@@ -220,23 +228,22 @@ void EncodeManager::logStats()
 
       ratio = (double)stats[i][j].equivalent / stats[i][j].bytes;
 
-      siPrefix(stats[i][j].rects, "rects", a, sizeof(a));
-      siPrefix(stats[i][j].pixels, "pixels", b, sizeof(b));
-      vlog.info("    %s: %s, %s", encoderTypeName((EncoderType)j), a, b);
-      iecPrefix(stats[i][j].bytes, "B", a, sizeof(a));
+      vlog.info("    %s: %s, %s", encoderTypeName((EncoderType)j),
+                core::siPrefix(stats[i][j].rects, "rects").c_str(),
+                core::siPrefix(stats[i][j].pixels, "pixels").c_str());
       vlog.info("    %*s  %s (1:%g ratio)",
                 (int)strlen(encoderTypeName((EncoderType)j)), "",
-                a, ratio);
+                core::iecPrefix(stats[i][j].bytes, "B").c_str(), ratio);
     }
   }
 
   ratio = (double)equivalent / bytes;
 
-  siPrefix(rects, "rects", a, sizeof(a));
-  siPrefix(pixels, "pixels", b, sizeof(b));
-  vlog.info("  Total: %s, %s", a, b);
-  iecPrefix(bytes, "B", a, sizeof(a));
-  vlog.info("         %s (1:%g ratio)", a, ratio);
+  vlog.info("  Total: %s, %s",
+            core::siPrefix(rects, "rects").c_str(),
+            core::siPrefix(pixels, "pixels").c_str());
+  vlog.info("         %s (1:%g ratio)",
+            core::iecPrefix(bytes, "B").c_str(), ratio);
 }
 
 bool EncodeManager::supported(int encoding)
@@ -245,6 +252,7 @@ bool EncodeManager::supported(int encoding)
   case encodingRaw:
   case encodingRRE:
   case encodingHextile:
+  case encodingJPEG:
   case encodingZRLE:
   case encodingTight:
     return true;
@@ -253,12 +261,12 @@ bool EncodeManager::supported(int encoding)
   }
 }
 
-bool EncodeManager::needsLosslessRefresh(const Region& req)
+bool EncodeManager::needsLosslessRefresh(const core::Region& req)
 {
   return !lossyRegion.intersect(req).is_empty();
 }
 
-int EncodeManager::getNextLosslessRefresh(const Region& req)
+int EncodeManager::getNextLosslessRefresh(const core::Region& req)
 {
   // Do we have something we can send right away?
   if (!pendingRefreshRegion.intersect(req).is_empty())
@@ -270,12 +278,19 @@ int EncodeManager::getNextLosslessRefresh(const Region& req)
   return recentChangeTimer.getNextTimeout();
 }
 
-void EncodeManager::pruneLosslessRefresh(const Region& limits)
+void EncodeManager::pruneLosslessRefresh(const core::Region& limits)
 {
   lossyRegion.assign_intersect(limits);
   pendingRefreshRegion.assign_intersect(limits);
 }
 
+void EncodeManager::forceRefresh(const core::Region& req)
+{
+  lossyRegion.assign_union(req);
+  if (!recentChangeTimer.isStarted())
+    pendingRefreshRegion.assign_union(req);
+}
+
 void EncodeManager::writeUpdate(const UpdateInfo& ui, const PixelBuffer* pb,
                                 const RenderedCursor* renderedCursor)
 {
@@ -287,15 +302,16 @@ void EncodeManager::writeUpdate(const UpdateInfo& ui, const PixelBuffer* pb,
     recentChangeTimer.start(RecentChangeTimeout);
 }
 
-void EncodeManager::writeLosslessRefresh(const Region& req, const PixelBuffer* pb,
+void EncodeManager::writeLosslessRefresh(const core::Region& req,
+                                         const PixelBuffer* pb,
                                          const RenderedCursor* renderedCursor,
                                          size_t maxUpdateSize)
 {
   doUpdate(false, getLosslessRefresh(req, maxUpdateSize),
-           Region(), Point(), pb, renderedCursor);
+           {}, {}, pb, renderedCursor);
 }
 
-bool EncodeManager::handleTimeout(Timer* t)
+void EncodeManager::handleTimeout(core::Timer* t)
 {
   if (t == &recentChangeTimer) {
     // Any lossy region that wasn't recently updated can
@@ -305,19 +321,19 @@ bool EncodeManager::handleTimeout(Timer* t)
 
     // Will there be more to do? (i.e. do we need another round)
     if (!lossyRegion.subtract(pendingRefreshRegion).is_empty())
-      return true;
+      t->repeat();
   }
-
-  return false;
 }
 
-void EncodeManager::doUpdate(bool allowLossy, const Region& changed_,
-                             const Region& copied, const Point& copyDelta,
+void EncodeManager::doUpdate(bool allowLossy, const
+                             core::Region& changed_,
+                             const core::Region& copied,
+                             const core::Point& copyDelta,
                              const PixelBuffer* pb,
                              const RenderedCursor* renderedCursor)
 {
     int nRects;
-    Region changed, cursorRegion;
+    core::Region changed, cursorRegion;
 
     updates++;
 
@@ -332,7 +348,7 @@ void EncodeManager::doUpdate(bool allowLossy, const Region& changed_,
      * We need to render the cursor seperately as it has its own
      * magical pixel buffer, so split it out from the changed region.
      */
-    if (renderedCursor != NULL) {
+    if (renderedCursor != nullptr) {
       cursorRegion = changed.intersect(renderedCursor->getEffectiveRect());
       changed.assign_subtract(renderedCursor->getEffectiveRect());
     }
@@ -340,7 +356,9 @@ void EncodeManager::doUpdate(bool allowLossy, const Region& changed_,
     if (conn->client.supportsEncoding(pseudoEncodingLastRect))
       nRects = 0xFFFF;
     else {
-      nRects = copied.numRects();
+      nRects = 0;
+      if (conn->client.supportsEncoding(encodingCopyRect))
+        nRects += copied.numRects();
       nRects += computeNumRects(changed);
       nRects += computeNumRects(cursorRegion);
     }
@@ -368,21 +386,13 @@ void EncodeManager::prepareEncoders(bool allowLossy)
   enum EncoderClass solid, bitmap, bitmapRLE;
   enum EncoderClass indexed, indexedRLE, fullColour;
 
-  bool allowJPEG;
-
-  rdr::S32 preferred;
+  int32_t preferred;
 
   std::vector<int>::iterator iter;
 
   solid = bitmap = bitmapRLE = encoderRaw;
   indexed = indexedRLE = fullColour = encoderRaw;
 
-  allowJPEG = conn->client.pf().bpp >= 16;
-  if (!allowLossy) {
-    if (encoders[encoderTightJPEG]->losslessQuality == -1)
-      allowJPEG = false;
-  }
-
   // Try to respect the client's wishes
   preferred = conn->getPreferredEncoding();
   switch (preferred) {
@@ -395,7 +405,10 @@ void EncodeManager::prepareEncoders(bool allowLossy)
     bitmapRLE = indexedRLE = fullColour = encoderHextile;
     break;
   case encodingTight:
-    if (encoders[encoderTightJPEG]->isSupported() && allowJPEG)
+    // Prefer the plain JPEG encoder as it compresses better
+    if (encoders[encoderJPEG]->isSupported())
+      fullColour = encoderJPEG;
+    else if (encoders[encoderTightJPEG]->isSupported())
       fullColour = encoderTightJPEG;
     else
       fullColour = encoderTight;
@@ -407,12 +420,55 @@ void EncodeManager::prepareEncoders(bool allowLossy)
     bitmapRLE = indexedRLE = encoderZRLE;
     bitmap = indexed = encoderZRLE;
     break;
+  case encodingJPEG:
+    fullColour = encoderJPEG;
+    break;
+  }
+
+  // JPEG is the only encoder that can reduce things to grayscale
+  if ((conn->client.subsampling == subsampleGray) && allowLossy) {
+    if (encoders[encoderJPEG]->isSupported()) {
+      solid = bitmap = bitmapRLE = encoderJPEG;
+      indexed = indexedRLE = fullColour = encoderJPEG;
+    } else if (encoders[encoderTightJPEG]->isSupported()) {
+      solid = bitmap = bitmapRLE = encoderTightJPEG;
+      indexed = indexedRLE = fullColour = encoderTightJPEG;
+    }
+  }
+
+  // Can't use lossy encoders for lossless refresh
+  if (!allowLossy) {
+    if ((fullColour != encoderRaw) &&
+        (encoders[fullColour]->flags & EncoderLossy) &&
+        (encoders[fullColour]->losslessQuality == -1))
+      fullColour = encoderRaw;
+    if ((indexed != encoderRaw) &&
+        (encoders[indexed]->flags & EncoderLossy) &&
+        (encoders[indexed]->losslessQuality == -1))
+      indexed = encoderRaw;
+    if ((indexedRLE != encoderRaw) &&
+        (encoders[indexedRLE]->flags & EncoderLossy) &&
+        (encoders[indexedRLE]->losslessQuality == -1))
+      indexedRLE = encoderRaw;
+    if ((bitmap != encoderRaw) &&
+        (encoders[bitmap]->flags & EncoderLossy) &&
+        (encoders[bitmap]->losslessQuality == -1))
+      bitmap = encoderRaw;
+    if ((bitmapRLE != encoderRaw) &&
+        (encoders[bitmapRLE]->flags & EncoderLossy) &&
+        (encoders[bitmapRLE]->losslessQuality == -1))
+      bitmapRLE = encoderRaw;
   }
 
   // Any encoders still unassigned?
 
   if (fullColour == encoderRaw) {
-    if (encoders[encoderTightJPEG]->isSupported() && allowJPEG)
+    if (encoders[encoderJPEG]->isSupported() &&
+        (allowLossy || encoders[encoderJPEG]->losslessQuality != -1))
+      fullColour = encoderJPEG;
+    else if (encoders[encoderTightJPEG]->isSupported() &&
+             (allowLossy ||
+              encoders[encoderTightJPEG]->losslessQuality != -1))
       fullColour = encoderTightJPEG;
     else if (encoders[encoderZRLE]->isSupported())
       fullColour = encoderZRLE;
@@ -450,13 +506,6 @@ void EncodeManager::prepareEncoders(bool allowLossy)
       solid = encoderHextile;
   }
 
-  // JPEG is the only encoder that can reduce things to grayscale
-  if ((conn->client.subsampling == subsampleGray) &&
-      encoders[encoderTightJPEG]->isSupported() && allowLossy) {
-    solid = bitmap = bitmapRLE = encoderTightJPEG;
-    indexed = indexedRLE = fullColour = encoderTightJPEG;
-  }
-
   activeEncoders[encoderSolid] = solid;
   activeEncoders[encoderBitmap] = bitmap;
   activeEncoders[encoderBitmapRLE] = bitmapRLE;
@@ -476,19 +525,20 @@ void EncodeManager::prepareEncoders(bool allowLossy)
       encoder->setFineQualityLevel(conn->client.fineQualityLevel,
                                    conn->client.subsampling);
     } else {
-      int level = __rfbmax(conn->client.qualityLevel,
-                           encoder->losslessQuality);
-      encoder->setQualityLevel(level);
+      if (conn->client.qualityLevel < encoder->losslessQuality)
+        encoder->setQualityLevel(encoder->losslessQuality);
+      else
+        encoder->setQualityLevel(conn->client.qualityLevel);
       encoder->setFineQualityLevel(-1, subsampleUndefined);
     }
   }
 }
 
-Region EncodeManager::getLosslessRefresh(const Region& req,
-                                         size_t maxUpdateSize)
+core::Region EncodeManager::getLosslessRefresh(const core::Region& req,
+                                               size_t maxUpdateSize)
 {
-  std::vector<Rect> rects;
-  Region refresh;
+  std::vector<core::Rect> rects;
+  core::Region refresh;
   size_t area;
 
   // We make a conservative guess at the compression ratio at 2:1
@@ -501,7 +551,7 @@ Region EncodeManager::getLosslessRefresh(const Region& req,
   pendingRefreshRegion.intersect(req).get_rects(&rects);
   while (!rects.empty()) {
     size_t idx;
-    Rect rect;
+    core::Rect rect;
 
     // Grab a random rect so we don't keep damaging and restoring the
     // same rect over and over
@@ -515,17 +565,21 @@ Region EncodeManager::getLosslessRefresh(const Region& req,
       // Use the narrowest axis to avoid getting to thin rects
       if (rect.width() > rect.height()) {
         int width = (maxUpdateSize - area) / rect.height();
-        rect.br.x = rect.tl.x + __rfbmax(1, width);
+        if (width < 1)
+          width = 1;
+        rect.br.x = rect.tl.x + width;
       } else {
         int height = (maxUpdateSize - area) / rect.width();
-        rect.br.y = rect.tl.y + __rfbmax(1, height);
+        if (height < 1)
+          height = 1;
+        rect.br.y = rect.tl.y + height;
       }
-      refresh.assign_union(Region(rect));
+      refresh.assign_union(rect);
       break;
     }
 
     area += rect.area();
-    refresh.assign_union(Region(rect));
+    refresh.assign_union(rect);
 
     rects.erase(rects.begin() + idx);
   }
@@ -533,11 +587,11 @@ Region EncodeManager::getLosslessRefresh(const Region& req,
   return refresh;
 }
 
-int EncodeManager::computeNumRects(const Region& changed)
+int EncodeManager::computeNumRects(const core::Region& changed)
 {
   int numRects;
-  std::vector<Rect> rects;
-  std::vector<Rect>::const_iterator rect;
+  std::vector<core::Rect> rects;
+  std::vector<core::Rect>::const_iterator rect;
 
   numRects = 0;
   changed.get_rects(&rects);
@@ -567,7 +621,7 @@ int EncodeManager::computeNumRects(const Region& changed)
   return numRects;
 }
 
-Encoder *EncodeManager::startRect(const Rect& rect, int type)
+Encoder* EncodeManager::startRect(const core::Rect& rect, int type)
 {
   Encoder *encoder;
   int klass, equiv;
@@ -588,13 +642,13 @@ Encoder *EncodeManager::startRect(const Rect& rect, int type)
   if ((encoder->flags & EncoderLossy) &&
       ((encoder->losslessQuality == -1) ||
        (encoder->getQualityLevel() < encoder->losslessQuality)))
-    lossyRegion.assign_union(Region(rect));
+    lossyRegion.assign_union(rect);
   else
-    lossyRegion.assign_subtract(Region(rect));
+    lossyRegion.assign_subtract(rect);
 
   // This was either a rect getting refreshed, or a rect that just got
   // new content. Either way we should not try to refresh it anymore.
-  pendingRefreshRegion.assign_subtract(Region(rect));
+  pendingRefreshRegion.assign_subtract(rect);
 
   return encoder;
 }
@@ -612,12 +666,13 @@ void EncodeManager::endRect()
   stats[klass][activeType].bytes += length;
 }
 
-void EncodeManager::writeCopyRects(const Region& copied, const Point& delta)
+void EncodeManager::writeCopyRects(const core::Region& copied,
+                                   const core::Point& delta)
 {
-  std::vector<Rect> rects;
-  std::vector<Rect>::const_iterator rect;
+  std::vector<core::Rect> rects;
+  std::vector<core::Rect>::const_iterator rect;
 
-  Region lossyCopy;
+  core::Region lossyCopy;
 
   beforeLength = conn->getOutStream()->length();
 
@@ -646,20 +701,22 @@ void EncodeManager::writeCopyRects(const Region& copied, const Point& delta)
   pendingRefreshRegion.assign_subtract(copied);
 }
 
-void EncodeManager::writeSolidRects(Region *changed, const PixelBuffer* pb)
+void EncodeManager::writeSolidRects(core::Region* changed,
+                                    const PixelBuffer* pb)
 {
-  std::vector<Rect> rects;
-  std::vector<Rect>::const_iterator rect;
+  std::vector<core::Rect> rects;
+  std::vector<core::Rect>::const_iterator rect;
 
   changed->get_rects(&rects);
   for (rect = rects.begin(); rect != rects.end(); ++rect)
     findSolidRect(*rect, changed, pb);
 }
 
-void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
+void EncodeManager::findSolidRect(const core::Rect& rect,
+                                  core::Region* changed,
                                   const PixelBuffer* pb)
 {
-  Rect sr;
+  core::Rect sr;
   int dx, dy, dw, dh;
 
   // We start by finding a solid 16x16 block
@@ -671,18 +728,18 @@ void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
 
     for (dx = rect.tl.x; dx < rect.br.x; dx += SolidSearchBlock) {
       // We define it like this to guarantee alignment
-      rdr::U32 _buffer;
-      rdr::U8* colourValue = (rdr::U8*)&_buffer;
+      uint32_t _buffer;
+      uint8_t* colourValue = (uint8_t*)&_buffer;
 
       dw = SolidSearchBlock;
       if (dx + dw > rect.br.x)
         dw = rect.br.x - dx;
 
-      pb->getImage(colourValue, Rect(dx, dy, dx+1, dy+1));
+      pb->getImage(colourValue, {dx, dy, dx+1, dy+1});
 
       sr.setXYWH(dx, dy, dw, dh);
       if (checkSolidTile(sr, colourValue, pb)) {
-        Rect erb, erp;
+        core::Rect erb, erp;
 
         Encoder *encoder;
 
@@ -693,7 +750,7 @@ void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
         extendSolidAreaByBlock(sr, colourValue, pb, &erb);
 
         // Did we end up getting the entire rectangle?
-        if (erb.equals(rect))
+        if (erb == rect)
           erp = erb;
         else {
           // Don't bother with sending tiny rectangles
@@ -711,8 +768,8 @@ void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
           encoder->writeSolidRect(erp.width(), erp.height(),
                                   pb->getPF(), colourValue);
         } else {
-          rdr::U32 _buffer2;
-          rdr::U8* converted = (rdr::U8*)&_buffer2;
+          uint32_t _buffer2;
+          uint8_t* converted = (uint8_t*)&_buffer2;
 
           conn->client.pf().bufferFromBuffer(converted, pb->getPF(),
                                          colourValue, 1);
@@ -722,7 +779,7 @@ void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
         }
         endRect();
 
-        changed->assign_subtract(Region(erp));
+        changed->assign_subtract(erp);
 
         // Search remaining areas by recursion
         // FIXME: Is this the best way to divide things up?
@@ -753,15 +810,16 @@ void EncodeManager::findSolidRect(const Rect& rect, Region *changed,
   }
 }
 
-void EncodeManager::writeRects(const Region& changed, const PixelBuffer* pb)
+void EncodeManager::writeRects(const core::Region& changed,
+                               const PixelBuffer* pb)
 {
-  std::vector<Rect> rects;
-  std::vector<Rect>::const_iterator rect;
+  std::vector<core::Rect> rects;
+  std::vector<core::Rect>::const_iterator rect;
 
   changed.get_rects(&rects);
   for (rect = rects.begin(); rect != rects.end(); ++rect) {
     int w, h, sw, sh;
-    Rect sr;
+    core::Rect sr;
 
     w = rect->width();
     h = rect->height();
@@ -795,7 +853,8 @@ void EncodeManager::writeRects(const Region& changed, const PixelBuffer* pb)
   }
 }
 
-void EncodeManager::writeSubRect(const Rect& rect, const PixelBuffer *pb)
+void EncodeManager::writeSubRect(const core::Rect& rect,
+                                 const PixelBuffer* pb)
 {
   PixelBuffer *ppb;
 
@@ -822,7 +881,8 @@ void EncodeManager::writeSubRect(const Rect& rect, const PixelBuffer *pb)
   maxColours = rect.area()/divisor;
 
   // Special exception inherited from the Tight encoder
-  if (activeEncoders[encoderFullColour] == encoderTightJPEG) {
+  if ((activeEncoders[encoderFullColour] == encoderTightJPEG) ||
+      (activeEncoders[encoderFullColour] == encoderJPEG)) {
     if ((conn->client.compressLevel != -1) && (conn->client.compressLevel < 2))
       maxColours = 24;
     else
@@ -879,26 +939,39 @@ void EncodeManager::writeSubRect(const Rect& rect, const PixelBuffer *pb)
   endRect();
 }
 
-bool EncodeManager::checkSolidTile(const Rect& r, const rdr::U8* colourValue,
+bool EncodeManager::checkSolidTile(const core::Rect& r,
+                                   const uint8_t* colourValue,
                                    const PixelBuffer *pb)
 {
+  const uint8_t* buffer;
+  int stride;
+
+  buffer = pb->getBuffer(r, &stride);
+
   switch (pb->getPF().bpp) {
   case 32:
-    return checkSolidTile(r, *(const rdr::U32*)colourValue, pb);
+    return checkSolidTile(r.width(), r.height(),
+                          (const uint32_t*)buffer, stride,
+                          *(const uint32_t*)colourValue);
   case 16:
-    return checkSolidTile(r, *(const rdr::U16*)colourValue, pb);
+    return checkSolidTile(r.width(), r.height(),
+                          (const uint16_t*)buffer, stride,
+                          *(const uint16_t*)colourValue);
   default:
-    return checkSolidTile(r, *(const rdr::U8*)colourValue, pb);
+    return checkSolidTile(r.width(), r.height(),
+                          (const uint8_t*)buffer, stride,
+                          *(const uint8_t*)colourValue);
   }
 }
 
-void EncodeManager::extendSolidAreaByBlock(const Rect& r,
-                                           const rdr::U8* colourValue,
-                                           const PixelBuffer *pb, Rect* er)
+void EncodeManager::extendSolidAreaByBlock(const core::Rect& r,
+                                           const uint8_t* colourValue,
+                                           const PixelBuffer* pb,
+                                           core::Rect* er)
 {
   int dx, dy, dw, dh;
   int w_prev;
-  Rect sr;
+  core::Rect sr;
   int w_best = 0, h_best = 0;
 
   w_prev = r.width();
@@ -948,12 +1021,14 @@ void EncodeManager::extendSolidAreaByBlock(const Rect& r,
   er->br.y = er->tl.y + h_best;
 }
 
-void EncodeManager::extendSolidAreaByPixel(const Rect& r, const Rect& sr,
-                                           const rdr::U8* colourValue,
-                                           const PixelBuffer *pb, Rect* er)
+void EncodeManager::extendSolidAreaByPixel(const core::Rect& r,
+                                           const core::Rect& sr,
+                                           const uint8_t* colourValue,
+                                           const PixelBuffer* pb,
+                                           core::Rect* er)
 {
   int cx, cy;
-  Rect tr;
+  core::Rect tr;
 
   // Try to extend the area upwards.
   for (cy = sr.tl.y - 1; cy >= r.tl.y; cy--) {
@@ -988,15 +1063,15 @@ void EncodeManager::extendSolidAreaByPixel(const Rect& r, const Rect& sr,
   er->br.x = cx;
 }
 
-PixelBuffer* EncodeManager::preparePixelBuffer(const Rect& rect,
+PixelBuffer* EncodeManager::preparePixelBuffer(const core::Rect& rect,
                                                const PixelBuffer *pb,
                                                bool convert)
 {
-  const rdr::U8* buffer;
+  const uint8_t* buffer;
   int stride;
 
   // Do wo need to convert the data?
-  if (convert && !conn->client.pf().equal(pb->getPF())) {
+  if (convert && conn->client.pf() != pb->getPF()) {
     convertedPixelBuffer.setPF(conn->client.pf());
     convertedPixelBuffer.setSize(rect.width(), rect.height());
 
@@ -1022,7 +1097,7 @@ PixelBuffer* EncodeManager::preparePixelBuffer(const Rect& rect,
 bool EncodeManager::analyseRect(const PixelBuffer *pb,
                                 struct RectInfo *info, int maxColours)
 {
-  const rdr::U8* buffer;
+  const uint8_t* buffer;
   int stride;
 
   buffer = pb->getBuffer(pb->getRect(), &stride);
@@ -1030,40 +1105,100 @@ bool EncodeManager::analyseRect(const PixelBuffer *pb,
   switch (pb->getPF().bpp) {
   case 32:
     return analyseRect(pb->width(), pb->height(),
-                       (const rdr::U32*)buffer, stride,
+                       (const uint32_t*)buffer, stride,
                        info, maxColours);
   case 16:
     return analyseRect(pb->width(), pb->height(),
-                       (const rdr::U16*)buffer, stride,
+                       (const uint16_t*)buffer, stride,
                        info, maxColours);
   default:
     return analyseRect(pb->width(), pb->height(),
-                       (const rdr::U8*)buffer, stride,
+                       (const uint8_t*)buffer, stride,
                        info, maxColours);
   }
 }
 
 void EncodeManager::OffsetPixelBuffer::update(const PixelFormat& pf,
                                               int width, int height,
-                                              const rdr::U8* data_,
+                                              const uint8_t* data_,
                                               int stride_)
 {
   format = pf;
-  width_ = width;
-  height_ = height;
   // Forced cast. We never write anything though, so it should be safe.
-  data = (rdr::U8*)data_;
-  stride = stride_;
+  setBuffer(width, height, (uint8_t*)data_, stride_);
 }
 
-// Preprocessor generated, optimised methods
+uint8_t* EncodeManager::OffsetPixelBuffer::getBufferRW(const core::Rect& /*r*/, int* /*stride*/)
+{
+  throw std::logic_error("Invalid write attempt to OffsetPixelBuffer");
+}
 
-#define BPP 8
-#include "EncodeManagerBPP.cxx"
-#undef BPP
-#define BPP 16
-#include "EncodeManagerBPP.cxx"
-#undef BPP
-#define BPP 32
-#include "EncodeManagerBPP.cxx"
-#undef BPP
+template<class T>
+inline bool EncodeManager::checkSolidTile(int width, int height,
+                                          const T* buffer, int stride,
+                                          const T colourValue)
+{
+  int pad;
+
+  pad = stride - width;
+
+  while (height--) {
+    int width_ = width;
+    while (width_--) {
+      if (*buffer != colourValue)
+        return false;
+      buffer++;
+    }
+    buffer += pad;
+  }
+
+  return true;
+}
+
+template<class T>
+inline bool EncodeManager::analyseRect(int width, int height,
+                                       const T* buffer, int stride,
+                                       struct RectInfo *info, int maxColours)
+{
+  int pad;
+
+  T colour;
+  int count;
+
+  info->rleRuns = 0;
+  info->palette.clear();
+
+  pad = stride - width;
+
+  // For efficiency, we only update the palette on changes in colour
+  colour = buffer[0];
+  count = 0;
+  while (height--) {
+    int w_ = width;
+    while (w_--) {
+      if (*buffer != colour) {
+        if (!info->palette.insert(colour, count))
+          return false;
+        if (info->palette.size() > maxColours)
+          return false;
+
+        // FIXME: This doesn't account for switching lines
+        info->rleRuns++;
+
+        colour = *buffer;
+        count = 0;
+      }
+      buffer++;
+      count++;
+    }
+    buffer += pad;
+  }
+
+  // Make sure the final pixels also get counted
+  if (!info->palette.insert(colour, count))
+    return false;
+  if (info->palette.size() > maxColours)
+    return false;
+
+  return true;
+}
