@@ -25,9 +25,9 @@
 #ifndef __RFB_JPEGCOMPRESSOR_H__
 #define __RFB_JPEGCOMPRESSOR_H__
 
+#include <core/Rect.h>
+
 #include <rdr/MemOutStream.h>
-#include <rfb/PixelFormat.h>
-#include <rfb/Rect.h>
 
 struct jpeg_compress_struct;
 
@@ -36,6 +36,9 @@ struct JPEG_DEST_MGR;
 
 namespace rfb {
 
+  class PixelFormat;
+  struct Rect;
+
   class JpegCompressor : public rdr::MemOutStream {
 
   public:
@@ -43,18 +46,22 @@ namespace rfb {
     JpegCompressor(int bufferLen = 128*1024);
     virtual ~JpegCompressor();
 
-    void compress(const rdr::U8 *, int, const Rect&, const PixelFormat&, int, int);
+    void setQualityLevel(int level);
+    void setFineQualityLevel(int quality, int subsampling);
 
-    void writeBytes(const void*, int);
+    int getQualityLevel();
 
-    inline rdr::U8* getstart() { return start; }
+    void compress(const uint8_t*, int, const core::Rect&,
+                  const PixelFormat&);
 
-    inline int overrun(int itemSize, int nItems) {
-      return MemOutStream::overrun(itemSize, nItems);
-    }
+    void writeBytes(const uint8_t*, int);
 
   private:
 
+    int qualityLevel;
+    int fineQuality;
+    int fineSubsampling;
+
     struct jpeg_compress_struct *cinfo;
 
     struct JPEG_ERROR_MGR *err;
