@@ -24,21 +24,29 @@
 #error "This source should not be compiled without HAVE_NETTLE defined"
 #endif
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 
+#include <vector>
+
 #include <nettle/bignum.h>
 #include <nettle/sha1.h>
 #include <nettle/sha2.h>
 #include <nettle/base64.h>
 #include <nettle/asn1.h>
-#include <rfb/SSecurityRSAAES.h>
-#include <rfb/SConnection.h>
-#include <rfb/LogWriter.h>
-#include <rfb/Exception.h>
+
+#include <core/Exception.h>
+#include <core/LogWriter.h>
+
 #include <rdr/AESInStream.h>
 #include <rdr/AESOutStream.h>
+#include <rdr/RandomStream.h>
+
+#include <rfb/SSecurityRSAAES.h>
+#include <rfb/SConnection.h>
+#include <rfb/Exception.h>
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <rfb/UnixPasswordValidator.h>
 #endif
@@ -61,21 +69,24 @@ const size_t MaxKeyFileSize = 32 * 1024;
 
 using namespace rfb;
 
-StringParameter SSecurityRSAAES::keyFile
+core::StringParameter SSecurityRSAAES::keyFile
 ("RSAKey", "Path to the RSA key for the RSA-AES security types in "
-           "PEM format", "", ConfServer);
-BoolParameter SSecurityRSAAES::requireUsername
+           "PEM format", "");
+core::BoolParameter SSecurityRSAAES::requireUsername
 ("RequireUsername", "Require username for the RSA-AES security types",
- false, ConfServer);
+ false);
 
-SSecurityRSAAES::SSecurityRSAAES(SConnection* sc, rdr::U32 _secType,
+static core::LogWriter vlog("SSecurityRSAAES");
+
+SSecurityRSAAES::SSecurityRSAAES(SConnection* sc_, uint32_t _secType,
                                  int _keySize, bool _isAllEncrypted)
-  : SSecurity(sc), state(SendPublicKey),
+  : SSecurity(sc_), state(SendPublicKey),
     keySize(_keySize), isAllEncrypted(_isAllEncrypted), secType(_secType),
     serverKey(), clientKey(),
-    serverKeyN(NULL), serverKeyE(NULL), clientKeyN(NULL), clientKeyE(NULL),
-    accessRights(SConnection::AccessDefault),
-    rais(NULL), raos(NULL), rawis(NULL), rawos(NULL)
+    serverKeyN(nullptr), serverKeyE(nullptr),
+    clientKeyN(nullptr), clientKeyE(nullptr),
+    accessRights(AccessDefault),
+    rais(nullptr), raos(nullptr), rawis(nullptr), rawos(nullptr)
 {
   assert(keySize == 128 || keySize == 256);
 }
@@ -87,6 +98,19 @@ SSecurityRSAAES::~SSecurityRSAAES()
 
 void SSecurityRSAAES::cleanup()
 {
+  if (raos) {
+    try {
+      if (raos->hasBufferedData()) {
+        raos->cork(false);
+        raos->flush();
+        if (raos->hasBufferedData())
+          vlog.error("Failed to flush remaining socket data on close");
+      }
+    } catch (std::exception& e) {
+      vlog.error("Failed to flush remaining socket data on close: %s", e.what());
+    }
+  }
+
   if (serverKeyN)
     delete[] serverKeyN;
   if (serverKeyE)
@@ -107,7 +131,7 @@ void SSecurityRSAAES::cleanup()
     delete raos;
 }
 
-static inline ssize_t findSubstr(rdr::U8* data, size_t size, const char *pattern)
+static inline ssize_t findSubstr(uint8_t* data, size_t size, const char *pattern)
 {
   size_t patternLength = strlen(pattern);
   for (size_t i = 0; i + patternLength < size; ++i) {
@@ -121,8 +145,8 @@ next:
   return -1;
 }
 
-static bool loadPEM(rdr::U8* data, size_t size, const char *begin,
-                    const char *end, rdr::U8** der, size_t *derSize)
+static bool loadPEM(uint8_t* data, size_t size, const char *begin,
+                    const char *end, std::vector<uint8_t> *der)
 {
   ssize_t pos1 = findSubstr(data, size, begin);
   if (pos1 == -1)
@@ -134,53 +158,56 @@ static bool loadPEM(rdr::U8* data, size_t size, const char *begin,
   char *derBase64 = (char *)data + pos1;
   if (!base64Size)
     return false;
-  *der = new rdr::U8[BASE64_DECODE_LENGTH(base64Size)];
+  der->resize(BASE64_DECODE_LENGTH(base64Size));
   struct base64_decode_ctx ctx;
+  size_t derSize;
   base64_decode_init(&ctx);
-  if (!base64_decode_update(&ctx, derSize, *der, base64Size, derBase64))
+  if (!base64_decode_update(&ctx, &derSize, der->data(),
+                            base64Size, derBase64))
     return false;
   if (!base64_decode_final(&ctx))
     return false;
+  assert(derSize <= der->size());
+  der->resize(derSize);
   return true;
 }
 
 void SSecurityRSAAES::loadPrivateKey()
 {
-  FILE* file = fopen(keyFile.getData(), "rb");
+  FILE* file = fopen(keyFile, "rb");
   if (!file)
-    throw ConnFailedException("failed to open key file");
+    throw core::posix_error("Failed to open key file", errno);
   fseek(file, 0, SEEK_END);
   size_t size = ftell(file);
   if (size == 0 || size > MaxKeyFileSize) {
     fclose(file);
-    throw ConnFailedException("size of key file is zero or too big");
+    throw std::runtime_error("Size of key file is zero or too big");
   }
   fseek(file, 0, SEEK_SET);
-  rdr::U8Array data(size);
-  if (fread(data.buf, 1, size, file) != size) {
+  std::vector<uint8_t> data(size);
+  if (fread(data.data(), 1, data.size(), file) != size) {
     fclose(file);
-    throw ConnFailedException("failed to read key");
+    throw core::posix_error("Failed to read key", errno);
   }
   fclose(file);
 
-  rdr::U8Array der;
-  size_t derSize;
-  if (loadPEM(data.buf, size, "-----BEGIN RSA PRIVATE KEY-----\n",
-              "-----END RSA PRIVATE KEY-----", &der.buf, &derSize)) {
-    loadPKCS1Key(der.buf, derSize);
+  std::vector<uint8_t> der;
+  if (loadPEM(data.data(), data.size(),
+              "-----BEGIN RSA PRIVATE KEY-----\n",
+              "-----END RSA PRIVATE KEY-----", &der)) {
+    loadPKCS1Key(der.data(), der.size());
     return;
   }
-  if (der.buf)
-    delete[] der.takeBuf();
-  if (loadPEM(data.buf, size, "-----BEGIN PRIVATE KEY-----\n",
-                     "-----END PRIVATE KEY-----", &der.buf, &derSize)) {
-    loadPKCS8Key(der.buf, derSize);
+  if (loadPEM(data.data(), data.size(),
+              "-----BEGIN PRIVATE KEY-----\n",
+              "-----END PRIVATE KEY-----", &der)) {
+    loadPKCS8Key(der.data(), der.size());
     return;
   }
-  throw ConnFailedException("failed to import key");
+  throw std::runtime_error("Failed to import key");
 }
 
-void SSecurityRSAAES::loadPKCS1Key(const rdr::U8* data, size_t size)
+void SSecurityRSAAES::loadPKCS1Key(const uint8_t* data, size_t size)
 {
   struct rsa_public_key pub;
   rsa_private_key_init(&serverKey);
@@ -188,17 +215,17 @@ void SSecurityRSAAES::loadPKCS1Key(const rdr::U8* data, size_t size)
   if (!rsa_keypair_from_der(&pub, &serverKey, 0, size, data)) {
     rsa_private_key_clear(&serverKey);
     rsa_public_key_clear(&pub);
-    throw ConnFailedException("failed to import key");
+    throw std::runtime_error("Failed to import key");
   }
   serverKeyLength = serverKey.size * 8;
-  serverKeyN = new rdr::U8[serverKey.size];
-  serverKeyE = new rdr::U8[serverKey.size];
+  serverKeyN = new uint8_t[serverKey.size];
+  serverKeyE = new uint8_t[serverKey.size];
   nettle_mpz_get_str_256(serverKey.size, serverKeyN, pub.n);
   nettle_mpz_get_str_256(serverKey.size, serverKeyE, pub.e);
   rsa_public_key_clear(&pub);
 }
 
-void SSecurityRSAAES::loadPKCS8Key(const rdr::U8* data, size_t size)
+void SSecurityRSAAES::loadPKCS8Key(const uint8_t* data, size_t size)
 {
   struct asn1_der_iterator i, j;
   uint32_t version;
@@ -228,7 +255,7 @@ void SSecurityRSAAES::loadPKCS8Key(const rdr::U8* data, size_t size)
   loadPKCS1Key(i.data, i.length);
   return;
 failed:
-  throw ConnFailedException("failed to import key");
+  throw std::runtime_error("Failed to import key");
 }
 
 bool SSecurityRSAAES::processMsg()
@@ -238,38 +265,39 @@ bool SSecurityRSAAES::processMsg()
       loadPrivateKey();
       writePublicKey();
       state = ReadPublicKey;
-      // fall through
+      /* fall through */
     case ReadPublicKey:
-      if (readPublicKey()) {
-        writeRandom();
-        state = ReadRandom;
-      }
-      return false;
+      if (!readPublicKey())
+        return false;
+      writeRandom();
+      state = ReadRandom;
+      /* fall through */
     case ReadRandom:
-      if (readRandom()) {
-        setCipher();
-        writeHash();
-        state = ReadHash;
-      }
-      return false;
+      if (!readRandom())
+        return false;
+      setCipher();
+      writeHash();
+      state = ReadHash;
+      /* fall through */
     case ReadHash:
-      if (readHash()) {
-        clearSecrets();
-        writeSubtype();
-        state = ReadCredentials;
-      }
-      return false;
+      if (!readHash())
+        return false;
+      clearSecrets();
+      writeSubtype();
+      state = ReadCredentials;
+      /* fall through */
     case ReadCredentials:
-      if (readCredentials()) {
-        if (requireUsername)
-          verifyUserPass();
-        else
-          verifyPass();
-        return true;
-      }
-      return false;
+      if (!readCredentials())
+        return false;
+      if (requireUsername)
+        verifyUserPass();
+      else
+        verifyPass();
+      return true;
   }
-  assert(!"unreachable");
+
+  throw std::logic_error("Invalid state");
+
   return false;
 }
 
@@ -290,22 +318,22 @@ bool SSecurityRSAAES::readPublicKey()
   is->setRestorePoint();
   clientKeyLength = is->readU32();
   if (clientKeyLength < MinKeyLength)
-    throw ConnFailedException("client key is too short");
+    throw protocol_error("Client key is too short");
   if (clientKeyLength > MaxKeyLength)
-    throw ConnFailedException("client key is too long");
+    throw protocol_error("Client key is too long");
   size_t size = (clientKeyLength + 7) / 8;
   if (!is->hasDataOrRestore(size * 2))
     return false;
   is->clearRestorePoint();
-  clientKeyE = new rdr::U8[size];
-  clientKeyN = new rdr::U8[size];
+  clientKeyE = new uint8_t[size];
+  clientKeyN = new uint8_t[size];
   is->readBytes(clientKeyN, size);
   is->readBytes(clientKeyE, size);
   rsa_public_key_init(&clientKey);
   nettle_mpz_set_str_256_u(clientKey.n, size, clientKeyN);
   nettle_mpz_set_str_256_u(clientKey.e, size, clientKeyE);
   if (!rsa_public_key_prepare(&clientKey))
-    throw ConnFailedException("client key is invalid");
+    throw protocol_error("Client key is invalid");
   return true;
 }
 
@@ -313,15 +341,16 @@ static void random_func(void* ctx, size_t length, uint8_t* dst)
 {
   rdr::RandomStream* rs = (rdr::RandomStream*)ctx;
   if (!rs->hasData(length))
-    throw ConnFailedException("failed to encrypt random");
+    throw std::runtime_error("Failed to encrypt random");
   rs->readBytes(dst, length);
 }
 
 void SSecurityRSAAES::writeRandom()
 {
+  rdr::RandomStream rs;
   rdr::OutStream* os = sc->getOutStream();
   if (!rs.hasData(keySize / 8))
-    throw ConnFailedException("failed to generate random");
+    throw std::runtime_error("Failed to generate random");
   rs.readBytes(serverRandom, keySize / 8);
   mpz_t x;
   mpz_init(x);
@@ -335,9 +364,9 @@ void SSecurityRSAAES::writeRandom()
   }
   if (!res) {
     mpz_clear(x);
-    throw ConnFailedException("failed to encrypt random");
+    throw std::runtime_error("Failed to encrypt random");
   }
-  rdr::U8* buffer = new rdr::U8[clientKey.size];
+  uint8_t* buffer = new uint8_t[clientKey.size];
   nettle_mpz_get_str_256(clientKey.size, buffer, x);
   mpz_clear(x);
   os->writeU16(clientKey.size);
@@ -354,11 +383,11 @@ bool SSecurityRSAAES::readRandom()
   is->setRestorePoint();
   size_t size = is->readU16();
   if (size != serverKey.size)
-    throw ConnFailedException("server key length doesn't match");
+    throw protocol_error("Server key length doesn't match");
   if (!is->hasDataOrRestore(size))
     return false;
   is->clearRestorePoint();
-  rdr::U8* buffer = new rdr::U8[size];
+  uint8_t* buffer = new uint8_t[size];
   is->readBytes(buffer, size);
   size_t randomSize = keySize / 8;
   mpz_t x;
@@ -367,7 +396,7 @@ bool SSecurityRSAAES::readRandom()
   if (!rsa_decrypt(&serverKey, &randomSize, clientRandom, x) ||
     randomSize != (size_t)keySize / 8) {
     mpz_clear(x);
-    throw ConnFailedException("failed to decrypt client random");
+    throw protocol_error("Failed to decrypt client random");
   }
   mpz_clear(x);
   return true;
@@ -377,7 +406,7 @@ void SSecurityRSAAES::setCipher()
 {
   rawis = sc->getInStream();
   rawos = sc->getOutStream();
-  rdr::U8 key[32];
+  uint8_t key[32];
   if (keySize == 128) {
     struct sha1_ctx ctx;
     sha1_init(&ctx);
@@ -409,20 +438,20 @@ void SSecurityRSAAES::setCipher()
 
 void SSecurityRSAAES::writeHash()
 {
-  rdr::U8 hash[32];
+  uint8_t hash[32];
   size_t len = serverKeyLength;
-  rdr::U8 lenServerKey[4] = {
-    (rdr::U8)((len & 0xff000000) >> 24),
-    (rdr::U8)((len & 0xff0000) >> 16),
-    (rdr::U8)((len & 0xff00) >> 8),
-    (rdr::U8)(len & 0xff)
+  uint8_t lenServerKey[4] = {
+    (uint8_t)((len & 0xff000000) >> 24),
+    (uint8_t)((len & 0xff0000) >> 16),
+    (uint8_t)((len & 0xff00) >> 8),
+    (uint8_t)(len & 0xff)
   };
   len = clientKeyLength;
-  rdr::U8 lenClientKey[4] = {
-    (rdr::U8)((len & 0xff000000) >> 24),
-    (rdr::U8)((len & 0xff0000) >> 16),
-    (rdr::U8)((len & 0xff00) >> 8),
-    (rdr::U8)(len & 0xff)
+  uint8_t lenClientKey[4] = {
+    (uint8_t)((len & 0xff000000) >> 24),
+    (uint8_t)((len & 0xff0000) >> 16),
+    (uint8_t)((len & 0xff00) >> 8),
+    (uint8_t)(len & 0xff)
   };
   int hashSize;
   if (keySize == 128) {
@@ -454,25 +483,25 @@ void SSecurityRSAAES::writeHash()
 
 bool SSecurityRSAAES::readHash()
 {
-  rdr::U8 hash[32];
-  rdr::U8 realHash[32];
+  uint8_t hash[32];
+  uint8_t realHash[32];
   int hashSize = keySize == 128 ? 20 : 32;
   if (!rais->hasData(hashSize))
     return false;
   rais->readBytes(hash, hashSize);
   size_t len = serverKeyLength;
-  rdr::U8 lenServerKey[4] = {
-    (rdr::U8)((len & 0xff000000) >> 24),
-    (rdr::U8)((len & 0xff0000) >> 16),
-    (rdr::U8)((len & 0xff00) >> 8),
-    (rdr::U8)(len & 0xff)
+  uint8_t lenServerKey[4] = {
+    (uint8_t)((len & 0xff000000) >> 24),
+    (uint8_t)((len & 0xff0000) >> 16),
+    (uint8_t)((len & 0xff00) >> 8),
+    (uint8_t)(len & 0xff)
   };
   len = clientKeyLength;
-  rdr::U8 lenClientKey[4] = {
-    (rdr::U8)((len & 0xff000000) >> 24),
-    (rdr::U8)((len & 0xff0000) >> 16),
-    (rdr::U8)((len & 0xff00) >> 8),
-    (rdr::U8)(len & 0xff)
+  uint8_t lenClientKey[4] = {
+    (uint8_t)((len & 0xff000000) >> 24),
+    (uint8_t)((len & 0xff0000) >> 16),
+    (uint8_t)((len & 0xff00) >> 8),
+    (uint8_t)(len & 0xff)
   };
   if (keySize == 128) {
     struct sha1_ctx ctx;
@@ -496,7 +525,7 @@ bool SSecurityRSAAES::readHash()
     sha256_digest(&ctx, hashSize, realHash);
   }
   if (memcmp(hash, realHash, hashSize) != 0)
-    throw ConnFailedException("hash doesn't match");
+    throw protocol_error("Hash doesn't match");
   return true;
 }
 
@@ -510,10 +539,10 @@ void SSecurityRSAAES::clearSecrets()
   delete[] serverKeyE;
   delete[] clientKeyN;
   delete[] clientKeyE;
-  serverKeyN = NULL;
-  serverKeyE = NULL;
-  clientKeyN = NULL;
-  clientKeyE = NULL;
+  serverKeyN = nullptr;
+  serverKeyE = nullptr;
+  clientKeyN = nullptr;
+  clientKeyE = nullptr;
   memset(serverRandom, 0, sizeof(serverRandom));
   memset(clientRandom, 0, sizeof(clientRandom));
 }
@@ -529,25 +558,19 @@ void SSecurityRSAAES::writeSubtype()
 
 bool SSecurityRSAAES::readCredentials()
 {
-  rais->setRestorePoint();
   if (!rais->hasData(1))
     return false;
-  rdr::U8 lenUsername = rais->readU8();
+  rais->setRestorePoint();
+  uint8_t lenUsername = rais->readU8();
   if (!rais->hasDataOrRestore(lenUsername + 1))
     return false;
-  if (!username.buf) {
-    username.replaceBuf(new char[lenUsername + 1]);
-    rais->readBytes(username.buf, lenUsername);
-    username.buf[lenUsername] = 0;
-  } else {
-    rais->skip(lenUsername);
-  }
-  rdr::U8 lenPassword = rais->readU8();
+  rais->readBytes((uint8_t*)username, lenUsername);
+  username[lenUsername] = 0;
+  uint8_t lenPassword = rais->readU8();
   if (!rais->hasDataOrRestore(lenPassword))
     return false;
-  password.replaceBuf(new char[lenPassword + 1]);
-  rais->readBytes(password.buf, lenPassword);
-  password.buf[lenPassword] = 0;
+  rais->readBytes((uint8_t*)password, lenPassword);
+  password[lenPassword] = 0;
   rais->clearRestorePoint();
   return true;
 }
@@ -560,39 +583,40 @@ void SSecurityRSAAES::verifyUserPass()
 #elif !defined(__APPLE__)
   UnixPasswordValidator *valid = new UnixPasswordValidator();
 #endif
-  if (!valid->validate(sc, username.buf, password.buf)) {
+  std::string msg = "Authentication failed";
+  if (!valid->validate(sc, username, password, msg)) {
     delete valid;
-    throw AuthFailureException("invalid password or username");
+    throw auth_error(msg);
   }
   delete valid;
 #else
-  throw AuthFailureException("No password validator configured");
+  throw std::logic_error("No password validator configured");
 #endif
 }
 
 void SSecurityRSAAES::verifyPass()
 {
   VncAuthPasswdGetter* pg = &SSecurityVncAuth::vncAuthPasswd;
-  PlainPasswd passwd, passwdReadOnly;
+  std::string passwd, passwdReadOnly;
   pg->getVncAuthPasswd(&passwd, &passwdReadOnly);
 
-  if (!passwd.buf)
-    throw AuthFailureException("No password configured for VNC Auth");
+  if (passwd.empty())
+    throw std::runtime_error("No password configured");
 
-  if (strcmp(password.buf, passwd.buf) == 0) {
-    accessRights = SConnection::AccessDefault;
+  if (password == passwd) {
+    accessRights = AccessDefault;
     return;
   }
 
-  if (passwdReadOnly.buf && strcmp(password.buf, passwdReadOnly.buf) == 0) {
-    accessRights = SConnection::AccessView;
+  if (!passwdReadOnly.empty() && password == passwdReadOnly) {
+    accessRights = AccessView;
     return;
   }
 
-  throw AuthFailureException();
+  throw auth_error("Authentication failed");
 }
 
 const char* SSecurityRSAAES::getUserName() const
 {
-  return username.buf;
+  return username;
 }
