@@ -27,23 +27,26 @@
 
 #include <vector>
 
+#include <core/string.h>
+
 #include <rdr/InStream.h>
 #include <rdr/MemInStream.h>
 #include <rdr/OutStream.h>
 
 #include <rfb/ServerParams.h>
 #include <rfb/Exception.h>
+#include <rfb/JpegDecompressor.h>
 #include <rfb/PixelBuffer.h>
 #include <rfb/TightConstants.h>
 #include <rfb/TightDecoder.h>
-#include <rfb/util.h>
 
 using namespace rfb;
 
 static const int TIGHT_MAX_WIDTH = 2048;
 static const int TIGHT_MIN_TO_COMPRESS = 12;
 
-TightDecoder::TightDecoder() : Decoder(DecoderPartiallyOrdered)
+TightDecoder::TightDecoder()
+  : Decoder(DecoderPartiallyOrdered), readState(IDLE)
 {
 }
 
@@ -51,73 +54,87 @@ TightDecoder::~TightDecoder()
 {
 }
 
-bool TightDecoder::readRect(const Rect& r, rdr::InStream* is,
+bool TightDecoder::readRect(const core::Rect& r, rdr::InStream* is,
                             const ServerParams& server, rdr::OutStream* os)
 {
-  uint8_t comp_ctl;
+  if (readState == IDLE) {
+    uint8_t comp_ctl;
 
-  if (!is->hasData(1))
-    return false;
+    if (r.width() > TIGHT_MAX_WIDTH)
+      throw protocol_error(core::format(
+        "TightDecoder: Too large rectangle (%d pixels)", r.width()));
 
-  is->setRestorePoint();
+    if (!is->hasData(1))
+      return false;
 
-  comp_ctl = is->readU8();
-  os->writeU8(comp_ctl);
+    comp_ctl = is->readU8();
+    os->writeU8(comp_ctl);
 
-  comp_ctl >>= 4;
+    comp_ctl >>= 4;
+
+    // Quit on unsupported compression type.
+    if (comp_ctl > tightMaxSubencoding)
+      throw protocol_error("TightDecoder: Bad subencoding value received");
+
+    if (comp_ctl == tightFill)
+      readState = FILL;
+    else if (comp_ctl == tightJpeg)
+      readState = JPEG;
+    else {
+      readPalSize = 0;
+      if ((comp_ctl & tightExplicitFilter) != 0)
+        readState = FILTER;
+      else
+        readState = PIXELS;
+    }
+  }
 
   // "Fill" compression type.
-  if (comp_ctl == tightFill) {
+  if (readState == FILL) {
     if (server.pf().is888()) {
-      if (!is->hasDataOrRestore(3))
+      if (!is->hasData(3))
         return false;
       os->copyBytes(is, 3);
     } else {
-      if (!is->hasDataOrRestore(server.pf().bpp/8))
+      if (!is->hasData(server.pf().bpp/8))
         return false;
       os->copyBytes(is, server.pf().bpp/8);
     }
-    is->clearRestorePoint();
+    readState = IDLE;
     return true;
   }
 
   // "JPEG" compression type.
-  if (comp_ctl == tightJpeg) {
+  if (readState == JPEG) {
     uint32_t len;
 
     // FIXME: Might be less than 3 bytes
-    if (!is->hasDataOrRestore(3))
+    if (!is->hasData(3))
       return false;
 
+    is->setRestorePoint();
+
     len = readCompact(is);
-    os->writeOpaque32(len);
 
     if (!is->hasDataOrRestore(len))
       return false;
 
+    os->writeOpaque32(len);
     os->copyBytes(is, len);
 
     is->clearRestorePoint();
 
+    readState = IDLE;
     return true;
   }
 
-  // Quit on unsupported compression type.
-  if (comp_ctl > tightMaxSubencoding)
-    throw protocol_error("TightDecoder: Bad subencoding value received");
-
   // "Basic" compression type.
 
-  int palSize = 0;
-
-  if (r.width() > TIGHT_MAX_WIDTH)
-    throw protocol_error(format("TightDecoder: Too large rectangle (%d pixels)", r.width()));
-
   // Possible palette
-  if ((comp_ctl & tightExplicitFilter) != 0) {
+  if (readState == FILTER) {
     uint8_t filterId;
 
-    if (!is->hasDataOrRestore(1))
+    if (!is->hasData(1))
       return false;
 
     filterId = is->readU8();
@@ -125,37 +142,52 @@ bool TightDecoder::readRect(const Rect& r, rdr::InStream* is,
 
     switch (filterId) {
     case tightFilterPalette:
-      if (!is->hasDataOrRestore(1))
-        return false;
-
-      palSize = is->readU8() + 1;
-      os->writeU8(palSize - 1);
-
-      if (server.pf().is888()) {
-        if (!is->hasDataOrRestore(palSize * 3))
-          return false;
-        os->copyBytes(is, palSize * 3);
-      } else {
-        if (!is->hasDataOrRestore(palSize * server.pf().bpp/8))
-          return false;
-        os->copyBytes(is, palSize * server.pf().bpp/8);
-      }
+      readState = PALETTE;
       break;
     case tightFilterGradient:
       if (server.pf().bpp == 8)
         throw protocol_error("TightDecoder: Invalid BPP for gradient filter");
+      readState = PIXELS;
       break;
     case tightFilterCopy:
+      readState = PIXELS;
       break;
     default:
       throw protocol_error("TightDecoder: Unknown filter code received");
     }
   }
 
+  if (readState == PALETTE) {
+    if (!is->hasData(1))
+      return false;
+
+    is->setRestorePoint();
+
+    readPalSize = is->readU8() + 1;
+
+    if (server.pf().is888()) {
+      if (!is->hasDataOrRestore(readPalSize * 3))
+        return false;
+      os->writeU8(readPalSize - 1);
+      os->copyBytes(is, readPalSize * 3);
+    } else {
+      if (!is->hasDataOrRestore(readPalSize * server.pf().bpp/8))
+        return false;
+      os->writeU8(readPalSize - 1);
+      os->copyBytes(is, readPalSize * server.pf().bpp/8);
+    }
+
+    is->clearRestorePoint();
+
+    readState = PIXELS;
+  }
+
+  assert(readState == PIXELS);
+
   size_t rowSize, dataSize;
 
-  if (palSize != 0) {
-    if (palSize <= 2)
+  if (readPalSize != 0) {
+    if (readPalSize <= 2)
       rowSize = (r.width() + 7) / 8;
     else
       rowSize = r.width();
@@ -168,34 +200,37 @@ bool TightDecoder::readRect(const Rect& r, rdr::InStream* is,
   dataSize = r.height() * rowSize;
 
   if (dataSize < TIGHT_MIN_TO_COMPRESS) {
-    if (!is->hasDataOrRestore(dataSize))
+    if (!is->hasData(dataSize))
       return false;
     os->copyBytes(is, dataSize);
   } else {
     uint32_t len;
 
     // FIXME: Might be less than 3 bytes
-    if (!is->hasDataOrRestore(3))
+    if (!is->hasData(3))
       return false;
 
+    is->setRestorePoint();
+
     len = readCompact(is);
-    os->writeOpaque32(len);
 
     if (!is->hasDataOrRestore(len))
       return false;
 
+    os->writeOpaque32(len);
     os->copyBytes(is, len);
+
+    is->clearRestorePoint();
   }
 
-  is->clearRestorePoint();
-
+  readState = IDLE;
   return true;
 }
 
-bool TightDecoder::doRectsConflict(const Rect& /*rectA*/,
+bool TightDecoder::doRectsConflict(const core::Rect& /*rectA*/,
                                    const uint8_t* bufferA,
                                    size_t buflenA,
-                                   const Rect& /*rectB*/,
+                                   const core::Rect& /*rectB*/,
                                    const uint8_t* bufferB,
                                    size_t buflenB,
                                    const ServerParams& /*server*/)
@@ -220,7 +255,7 @@ bool TightDecoder::doRectsConflict(const Rect& /*rectA*/,
   return false;
 }
 
-void TightDecoder::decodeRect(const Rect& r, const uint8_t* buffer,
+void TightDecoder::decodeRect(const core::Rect& r, const uint8_t* buffer,
                               size_t buflen, const ServerParams& server,
                               ModifiablePixelBuffer* pb)
 {
@@ -506,7 +541,7 @@ uint32_t TightDecoder::readCompact(rdr::InStream* is)
 void
 TightDecoder::FilterGradient24(const uint8_t *inbuf,
                                const PixelFormat& pf, uint32_t* outbuf,
-                               int stride, const Rect& r)
+                               int stride, const core::Rect& r)
 {
   int x, y, c;
   uint8_t prevRow[TIGHT_MAX_WIDTH*3];
@@ -552,7 +587,7 @@ TightDecoder::FilterGradient24(const uint8_t *inbuf,
 template<class T>
 void TightDecoder::FilterGradient(const uint8_t* inbuf,
                                   const PixelFormat& pf, T* outbuf,
-                                  int stride, const Rect& r)
+                                  int stride, const core::Rect& r)
 {
   int x, y, c;
   static uint8_t prevRow[TIGHT_MAX_WIDTH*3];
@@ -606,7 +641,7 @@ void TightDecoder::FilterGradient(const uint8_t* inbuf,
 template<class T>
 void TightDecoder::FilterPalette(const T* palette, int palSize,
                                  const uint8_t* inbuf, T* outbuf,
-                                 int stride, const Rect& r)
+                                 int stride, const core::Rect& r)
 {
   // Indexed color
   int x, h = r.height(), w = r.width(), b, pad = stride - w;
