@@ -22,6 +22,13 @@
 #include <config.h>
 #endif
 
+#include <core/LogWriter.h>
+#include <core/string.h>
+#include <core/time.h>
+
+#include <rdr/FdInStream.h>
+#include <rdr/FdOutStream.h>
+
 #include <network/TcpSocket.h>
 
 #include <rfb/ComparingUpdateTracker.h>
@@ -29,12 +36,12 @@
 #include <rfb/Exception.h>
 #include <rfb/KeyRemapper.h>
 #include <rfb/KeysymStr.h>
-#include <rfb/LogWriter.h>
 #include <rfb/Security.h>
 #include <rfb/ServerCore.h>
 #include <rfb/SMsgWriter.h>
 #include <rfb/VNCServerST.h>
 #include <rfb/VNCSConnectionST.h>
+#include <rfb/encodings.h>
 #include <rfb/screenTypes.h>
 #include <rfb/fenceTypes.h>
 #include <rfb/ledStates.h>
@@ -42,13 +49,12 @@
 #define XK_MISCELLANY
 #define XK_XKB_KEYS
 #include <rfb/keysymdef.h>
-#include <rfb/util.h>
 
 using namespace rfb;
 
-static LogWriter vlog("VNCSConnST");
+static core::LogWriter vlog("VNCSConnST");
 
-static Cursor emptyCursor(0, 0, Point(0, 0), NULL);
+static Cursor emptyCursor(0, 0, {0, 0}, nullptr);
 
 VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
                                    bool reverse, AccessRights ar)
@@ -56,7 +62,7 @@ VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
     sock(s), reverseConnection(reverse),
     inProcessMessages(false),
     pendingSyncFence(false), syncFence(false), fenceFlags(0),
-    fenceDataLen(0), fenceData(NULL), congestionTimer(this),
+    fenceDataLen(0), fenceData(nullptr), congestionTimer(this),
     losslessTimer(this), server(server_),
     updateRenderedCursor(false), removeRenderedCursor(false),
     continuousUpdates(false), encodeManager(this), idleTimer(this),
@@ -69,9 +75,9 @@ VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
   if (rfb::Server::idleTimeout) {
     // minimum of 15 seconds while authenticating
     if (rfb::Server::idleTimeout < 15)
-      idleTimer.start(secsToMillis(15));
+      idleTimer.start(core::secsToMillis(15));
     else
-      idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
+      idleTimer.start(core::secsToMillis(rfb::Server::idleTimeout));
   }
 }
 
@@ -80,12 +86,9 @@ VNCSConnectionST::~VNCSConnectionST()
 {
   // If we reach here then VNCServerST is deleting us!
   if (!closeReason.empty())
-    vlog.info("closing %s: %s", peerEndpoint.c_str(),
+    vlog.info("Closing %s: %s", peerEndpoint.c_str(),
               closeReason.c_str());
 
-  // Release any mouse buttons
-  server->pointerEvent(this, server->getCursorPos(), 0);
-
   // Release any keys the client still had pressed
   while (!pressedKeys.empty()) {
     uint32_t keysym, keycode;
@@ -123,18 +126,7 @@ void VNCSConnectionST::close(const char* reason)
   if (closeReason.empty())
     closeReason = reason;
   else
-    vlog.debug("second close: %s (%s)", peerEndpoint.c_str(), reason);
-
-  try {
-    if (sock->outStream().hasBufferedData()) {
-      sock->outStream().cork(false);
-      sock->outStream().flush();
-      if (sock->outStream().hasBufferedData())
-        vlog.error("Failed to flush remaining socket data on close");
-    }
-  } catch (rdr::Exception& e) {
-    vlog.error("Failed to flush remaining socket data on close: %s", e.str());
-  }
+    vlog.debug("Second close: %s (%s)", peerEndpoint.c_str(), reason);
 
   // Just shutdown the socket and mark our state as closing.  Eventually the
   // calling code will call VNCServerST's removeSocket() method causing us to
@@ -149,8 +141,8 @@ bool VNCSConnectionST::init()
 {
   try {
     initialiseProtocol();
-  } catch (rdr::Exception& e) {
-    close(e.str());
+  } catch (std::exception& e) {
+    close(e.what());
     return false;
   }
   return true;
@@ -190,10 +182,10 @@ void VNCSConnectionST::processMessages()
     // We wait until now with this to aggregate responses and to give 
     // higher priority to user actions such as keyboard and pointer events.
     writeFramebufferUpdate();
-  } catch (rdr::EndOfStream&) {
+  } catch (rdr::end_of_stream&) {
     close("Clean disconnection");
-  } catch (rdr::Exception &e) {
-    close(e.str());
+  } catch (std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -206,15 +198,16 @@ void VNCSConnectionST::flushSocket()
     // delayed because of congestion.
     if (!sock->outStream().hasBufferedData())
       writeFramebufferUpdate();
-  } catch (rdr::Exception &e) {
-    close(e.str());
+  } catch (std::exception& e) {
+    close(e.what());
   }
 }
 
 void VNCSConnectionST::pixelBufferChange()
 {
   try {
-    if (!authenticated()) return;
+    if (state() != RFBSTATE_NORMAL)
+      return;
     if (client.width() && client.height() &&
         (server->getPixelBuffer()->width() != client.width() ||
          server->getPixelBuffer()->height() != client.height()))
@@ -228,11 +221,11 @@ void VNCSConnectionST::pixelBufferChange()
       //updates.intersect(server->pb->getRect());
       //
       //if (server->pb->width() > client.width())
-      //  updates.add_changed(Rect(client.width(), 0, server->pb->width(),
-      //                           server->pb->height()));
+      //  updates.add_changed({client.width(), 0, server->pb->width(),
+      //                       server->pb->height()});
       //if (server->pb->height() > client.height())
-      //  updates.add_changed(Rect(0, client.height(), client.width(),
-      //                           server->pb->height()));
+      //  updates.add_changed({0, client.height(), client.width(),
+      //                       server->pb->height()});
 
       damagedCursorRegion.assign_intersect(server->getPixelBuffer()->getRect());
 
@@ -248,15 +241,15 @@ void VNCSConnectionST::pixelBufferChange()
       }
 
       // Drop any lossy tracking that is now outside the framebuffer
-      encodeManager.pruneLosslessRefresh(Region(server->getPixelBuffer()->getRect()));
+      encodeManager.pruneLosslessRefresh(server->getPixelBuffer()->getRect());
     }
     // Just update the whole screen at the moment because we're too lazy to
     // work out what's actually changed.
     updates.clear();
     updates.add_changed(server->getPixelBuffer()->getRect());
     writeFramebufferUpdate();
-  } catch(rdr::Exception &e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -264,8 +257,8 @@ void VNCSConnectionST::writeFramebufferUpdateOrClose()
 {
   try {
     writeFramebufferUpdate();
-  } catch(rdr::Exception &e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -274,8 +267,8 @@ void VNCSConnectionST::screenLayoutChangeOrClose(uint16_t reason)
   try {
     screenLayoutChange(reason);
     writeFramebufferUpdate();
-  } catch(rdr::Exception &e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -283,8 +276,8 @@ void VNCSConnectionST::bellOrClose()
 {
   try {
     if (state() == RFBSTATE_NORMAL) writer()->writeBell();
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -293,8 +286,8 @@ void VNCSConnectionST::setDesktopNameOrClose(const char *name)
   try {
     setDesktopName(name);
     writeFramebufferUpdate();
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -303,8 +296,8 @@ void VNCSConnectionST::setCursorOrClose()
   try {
     setCursor();
     writeFramebufferUpdate();
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -313,8 +306,8 @@ void VNCSConnectionST::setLEDStateOrClose(unsigned int state)
   try {
     setLEDState(state);
     writeFramebufferUpdate();
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -322,11 +315,9 @@ void VNCSConnectionST::requestClipboardOrClose()
 {
   try {
     if (state() != RFBSTATE_NORMAL) return;
-    if (!accessCheck(AccessCutText)) return;
-    if (!rfb::Server::acceptCutText) return;
     requestClipboard();
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -334,11 +325,9 @@ void VNCSConnectionST::announceClipboardOrClose(bool available)
 {
   try {
     if (state() != RFBSTATE_NORMAL) return;
-    if (!accessCheck(AccessCutText)) return;
-    if (!rfb::Server::sendCutText) return;
     announceClipboard(available);
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -346,11 +335,19 @@ void VNCSConnectionST::sendClipboardDataOrClose(const char* data)
 {
   try {
     if (state() != RFBSTATE_NORMAL) return;
-    if (!accessCheck(AccessCutText)) return;
-    if (!rfb::Server::sendCutText) return;
     sendClipboardData(data);
-  } catch(rdr::Exception& e) {
-    close(e.str());
+  } catch(std::exception& e) {
+    close(e.what());
+  }
+}
+
+void VNCSConnectionST::desktopReadyOrClose()
+{
+  try {
+    if (state() != RFBSTATE_CLIENT_READY) return;
+    desktopReady();
+  } catch(std::exception& e) {
+    close(e.what());
   }
 }
 
@@ -409,31 +406,16 @@ bool VNCSConnectionST::needRenderedCursor()
   if (!client.supportsLocalCursor())
     return true;
   if ((server->getCursorPos() != pointerEventPos) &&
-      (time(0) - pointerEventTime) > 0)
+      (time(nullptr) - pointerEventTime) > 0)
     return true;
 
   return false;
 }
 
-
-void VNCSConnectionST::approveConnectionOrClose(bool accept,
-                                                const char* reason)
+void VNCSConnectionST::desktopReady()
 {
-  try {
-    approveConnection(accept, reason);
-  } catch (rdr::Exception& e) {
-    close(e.str());
-  }
-}
-
-
-
-// -=- Callbacks from SConnection
-
-void VNCSConnectionST::authSuccess()
-{
-  if (rfb::Server::idleTimeout)
-    idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
+  if (state() != RFBSTATE_CLIENT_READY)
+    return;
 
   // - Set the connection parameters appropriately
   client.setDimensions(server->getPixelBuffer()->width(),
@@ -450,6 +432,29 @@ void VNCSConnectionST::authSuccess()
 
   // - Mark the entire display as "dirty"
   updates.add_changed(server->getPixelBuffer()->getRect());
+
+  SConnection::desktopReady();
+}
+
+
+void VNCSConnectionST::approveConnectionOrClose(bool accept,
+                                                const char* reason)
+{
+  try {
+    approveConnection(accept, reason);
+  } catch (std::exception& e) {
+    close(e.what());
+  }
+}
+
+
+
+// -=- Callbacks from SConnection
+
+void VNCSConnectionST::authSuccess()
+{
+  if (rfb::Server::idleTimeout)
+    idleTimer.start(core::secsToMillis(rfb::Server::idleTimeout));
 }
 
 void VNCSConnectionST::queryConnection(const char* userName)
@@ -457,15 +462,18 @@ void VNCSConnectionST::queryConnection(const char* userName)
   server->queryConnection(this, userName);
 }
 
-void VNCSConnectionST::clientInit(bool shared)
+void VNCSConnectionST::clientReady(bool shared)
 {
   if (rfb::Server::idleTimeout)
-    idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
+    idleTimer.start(core::secsToMillis(rfb::Server::idleTimeout));
+
   if (rfb::Server::alwaysShared || reverseConnection) shared = true;
   if (!accessCheck(AccessNonShared)) shared = true;
   if (rfb::Server::neverShared) shared = false;
-  SConnection::clientInit(shared);
   server->clientReady(this, shared);
+
+  if (server->isDesktopReady())
+    desktopReady();
 }
 
 void VNCSConnectionST::setPixelFormat(const PixelFormat& pf)
@@ -475,15 +483,16 @@ void VNCSConnectionST::setPixelFormat(const PixelFormat& pf)
   pf.print(buffer, 256);
   vlog.info("Client pixel format %s", buffer);
   setCursor();
+  encodeManager.forceRefresh(server->getPixelBuffer()->getRect());
 }
 
-void VNCSConnectionST::pointerEvent(const Point& pos, int buttonMask)
+void VNCSConnectionST::pointerEvent(const core::Point& pos,
+                                    uint16_t buttonMask)
 {
   if (rfb::Server::idleTimeout)
-    idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
-  pointerEventTime = time(0);
+    idleTimer.start(core::secsToMillis(rfb::Server::idleTimeout));
+  pointerEventTime = time(nullptr);
   if (!accessCheck(AccessPtrEvents)) return;
-  if (!rfb::Server::acceptPointerEvents) return;
   pointerEventPos = pos;
   server->pointerEvent(this, pointerEventPos, buttonMask);
 }
@@ -496,12 +505,12 @@ public:
   ~VNCSConnectionSTShiftPresser() {
     if (pressed) {
       vlog.debug("Releasing fake Shift_L");
-      server->keyEvent(XK_Shift_L, 0, false);
+      server->keyEvent(XK_Shift_L, 0x2a, false);
     }
   }
   void press() {
     vlog.debug("Pressing fake Shift_L");
-    server->keyEvent(XK_Shift_L, 0, true);
+    server->keyEvent(XK_Shift_L, 0x2a, true);
     pressed = true;
   }
   VNCServerST* server;
@@ -514,8 +523,10 @@ void VNCSConnectionST::keyEvent(uint32_t keysym, uint32_t keycode, bool down) {
   uint32_t lookup;
 
   if (rfb::Server::idleTimeout)
-    idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
+    idleTimer.start(core::secsToMillis(rfb::Server::idleTimeout));
   if (!accessCheck(AccessKeyEvents)) return;
+  // FIXME: This check isn't strictly needed, but we get a lot of
+  //        confusing debug logging without it
   if (!rfb::Server::acceptKeyEvents) return;
 
   if (down)
@@ -550,8 +561,8 @@ void VNCSConnectionST::keyEvent(uint32_t keysym, uint32_t keycode, bool down) {
 
         if (lock == (uppercase == shift)) {
           vlog.debug("Inserting fake CapsLock to get in sync with client");
-          server->keyEvent(XK_Caps_Lock, 0, true);
-          server->keyEvent(XK_Caps_Lock, 0, false);
+          server->keyEvent(XK_Caps_Lock, 0x3a, true);
+          server->keyEvent(XK_Caps_Lock, 0x3a, false);
         }
       }
 
@@ -580,8 +591,8 @@ void VNCSConnectionST::keyEvent(uint32_t keysym, uint32_t keycode, bool down) {
           //
         } else if (lock == (number == shift)) {
           vlog.debug("Inserting fake NumLock to get in sync with client");
-          server->keyEvent(XK_Num_Lock, 0, true);
-          server->keyEvent(XK_Num_Lock, 0, false);
+          server->keyEvent(XK_Num_Lock, 0x45, true);
+          server->keyEvent(XK_Num_Lock, 0x45, false);
         }
       }
     }
@@ -617,27 +628,28 @@ void VNCSConnectionST::keyEvent(uint32_t keysym, uint32_t keycode, bool down) {
   server->keyEvent(keysym, keycode, down);
 }
 
-void VNCSConnectionST::framebufferUpdateRequest(const Rect& r,bool incremental)
+void VNCSConnectionST::framebufferUpdateRequest(const core::Rect& r,
+                                                bool incremental)
 {
-  Rect safeRect;
+  core::Rect safeRect;
 
   if (!accessCheck(AccessView)) return;
 
   SConnection::framebufferUpdateRequest(r, incremental);
 
   // Check that the client isn't sending crappy requests
-  if (!r.enclosed_by(Rect(0, 0, client.width(), client.height()))) {
+  if (!r.enclosed_by({0, 0, client.width(), client.height()})) {
     vlog.error("FramebufferUpdateRequest %dx%d at %d,%d exceeds framebuffer %dx%d",
                r.width(), r.height(), r.tl.x, r.tl.y,
                client.width(), client.height());
-    safeRect = r.intersect(Rect(0, 0, client.width(), client.height()));
+    safeRect = r.intersect({0, 0, client.width(), client.height()});
   } else {
     safeRect = r;
   }
 
   // Just update the requested region.
   // Framebuffer update will be sent a bit later, see processMessages().
-  Region reqRgn(safeRect);
+  core::Region reqRgn(safeRect);
   if (!incremental || !continuousUpdates)
     requested.assign_union(reqRgn);
 
@@ -668,8 +680,7 @@ void VNCSConnectionST::setDesktopSize(int fb_width, int fb_height,
   layout.print(buffer, sizeof(buffer));
   vlog.debug("%s", buffer);
 
-  if (!accessCheck(AccessSetDesktopSize) ||
-      !rfb::Server::acceptSetDesktopSize) {
+  if (!accessCheck(AccessSetDesktopSize)) {
     vlog.debug("Rejecting unauthorized framebuffer resize request");
     result = resultProhibited;
   } else {
@@ -690,7 +701,7 @@ void VNCSConnectionST::fence(uint32_t flags, unsigned len, const uint8_t data[])
       fenceFlags = flags & (fenceFlagBlockBefore | fenceFlagBlockAfter | fenceFlagSyncNext);
       fenceDataLen = len;
       delete [] fenceData;
-      fenceData = NULL;
+      fenceData = nullptr;
       if (len > 0) {
         fenceData = new uint8_t[len];
         memcpy(fenceData, data, len);
@@ -706,8 +717,10 @@ void VNCSConnectionST::fence(uint32_t flags, unsigned len, const uint8_t data[])
     return;
   }
 
-  if (len < 1)
+  if (len < 1) {
     vlog.error("Fence response of unexpected size received");
+    return;
+  }
 
   type = data[0];
 
@@ -726,10 +739,13 @@ void VNCSConnectionST::fence(uint32_t flags, unsigned len, const uint8_t data[])
 void VNCSConnectionST::enableContinuousUpdates(bool enable,
                                                int x, int y, int w, int h)
 {
-  Rect rect;
+  core::Rect rect;
+
+  if (!accessCheck(AccessView))
+    return;
 
   if (!client.supportsFence() || !client.supportsContinuousUpdates())
-    throw Exception("Client tried to enable continuous updates when not allowed");
+    throw protocol_error("Client tried to enable continuous updates when not allowed");
 
   continuousUpdates = enable;
 
@@ -745,21 +761,16 @@ void VNCSConnectionST::enableContinuousUpdates(bool enable,
 
 void VNCSConnectionST::handleClipboardRequest()
 {
-  if (!accessCheck(AccessCutText)) return;
   server->handleClipboardRequest(this);
 }
 
 void VNCSConnectionST::handleClipboardAnnounce(bool available)
 {
-  if (!accessCheck(AccessCutText)) return;
-  if (!rfb::Server::acceptCutText) return;
   server->handleClipboardAnnounce(this, available);
 }
 
 void VNCSConnectionST::handleClipboardData(const char* data)
 {
-  if (!accessCheck(AccessCutText)) return;
-  if (!rfb::Server::acceptCutText) return;
   server->handleClipboardData(this, data);
 }
 
@@ -800,14 +811,14 @@ void VNCSConnectionST::supportsLEDState()
   writer()->writeLEDState();
 }
 
-void VNCSConnectionST::handleTimeout(Timer* t)
+void VNCSConnectionST::handleTimeout(core::Timer* t)
 {
   try {
     if ((t == &congestionTimer) ||
         (t == &losslessTimer))
       writeFramebufferUpdate();
-  } catch (rdr::Exception& e) {
-    close(e.str());
+  } catch (std::exception& e) {
+    close(e.what());
   }
 
   if (t == &idleTimer)
@@ -931,7 +942,7 @@ void VNCSConnectionST::writeNoDataUpdate()
 
 void VNCSConnectionST::writeDataUpdate()
 {
-  Region req;
+  core::Region req;
   UpdateInfo ui;
   bool needNewUpdateInfo;
   const RenderedCursor *cursor;
@@ -956,7 +967,7 @@ void VNCSConnectionST::writeDataUpdate()
   // destination will be wrong, so add it to the changed region.
 
   if (!ui.copied.is_empty() && !damagedCursorRegion.is_empty()) {
-    Region bogusCopiedCursor;
+    core::Region bogusCopiedCursor;
 
     bogusCopiedCursor = damagedCursorRegion;
     bogusCopiedCursor.translate(ui.copy_delta);
@@ -1001,9 +1012,9 @@ void VNCSConnectionST::writeDataUpdate()
 
   // Does the client need a server-side rendered cursor?
 
-  cursor = NULL;
+  cursor = nullptr;
   if (needRenderedCursor()) {
-    Rect renderedCursorRect;
+    core::Rect renderedCursorRect;
 
     cursor = server->getRenderedCursor();
     renderedCursorRect = cursor->getEffectiveRect();
@@ -1043,7 +1054,7 @@ void VNCSConnectionST::writeDataUpdate()
 
 void VNCSConnectionST::writeLosslessRefresh()
 {
-  Region req, pending;
+  core::Region req, pending;
   const RenderedCursor *cursor;
 
   int nextRefresh, nextUpdate;
@@ -1083,7 +1094,7 @@ void VNCSConnectionST::writeLosslessRefresh()
 
   // Prepare the cursor in case it overlaps with a region getting
   // refreshed
-  cursor = NULL;
+  cursor = nullptr;
   if (needRenderedCursor())
     cursor = server->getRenderedCursor();
 
@@ -1118,15 +1129,12 @@ void VNCSConnectionST::writeLosslessRefresh()
 
 void VNCSConnectionST::screenLayoutChange(uint16_t reason)
 {
-  if (!authenticated())
+  if (state() != RFBSTATE_NORMAL)
     return;
 
   client.setDimensions(client.width(), client.height(),
                        server->getScreenLayout());
 
-  if (state() != RFBSTATE_NORMAL)
-    return;
-
   writer()->writeDesktopSize(reason);
 }
 
