@@ -17,7 +17,7 @@
  * USA.
  */
 
-// -=- Single-Threaded VNC Server implementation
+// -=- Single-Threaded VNC server implementation
 
 
 // Note about how sockets get closed:
@@ -55,25 +55,28 @@
 #include <assert.h>
 #include <stdlib.h>
 
+#include <core/LogWriter.h>
+#include <core/time.h>
+
+#include <rdr/FdOutStream.h>
+
 #include <network/Socket.h>
 
 #include <rfb/ComparingUpdateTracker.h>
-#include <rfb/Exception.h>
 #include <rfb/KeyRemapper.h>
 #include <rfb/KeysymStr.h>
-#include <rfb/LogWriter.h>
+#include <rfb/SDesktop.h>
 #include <rfb/Security.h>
 #include <rfb/ServerCore.h>
 #include <rfb/VNCServerST.h>
 #include <rfb/VNCSConnectionST.h>
-#include <rfb/util.h>
 #include <rfb/ledStates.h>
 
 
 using namespace rfb;
 
-static LogWriter slog("VNCServerST");
-static LogWriter connectionsLog("Connections");
+static core::LogWriter slog("VNCServerST");
+static core::LogWriter connectionsLog("Connections");
 
 //
 // -=- VNCServerST Implementation
@@ -83,29 +86,29 @@ static LogWriter connectionsLog("Connections");
 
 VNCServerST::VNCServerST(const char* name_, SDesktop* desktop_)
   : blHosts(&blacklist), desktop(desktop_), desktopStarted(false),
-    blockCounter(0), pb(0), ledState(ledUnknown),
-    name(name_), pointerClient(0), clipboardClient(0),
-    pointerClientTime(0),
-    comparer(0), cursor(new Cursor(0, 0, Point(), NULL)),
+    desktopStarting(false), blockCounter(0), pb(nullptr),
+    ledState(ledUnknown), name(name_), pointerClient(nullptr),
+    clipboardClient(nullptr), pointerClientTime(0),
+    comparer(nullptr), cursor(new Cursor(0, 0, {}, nullptr)),
     renderedCursorInvalid(false),
     keyRemapper(&KeyRemapper::defInstance),
     idleTimer(this), disconnectTimer(this), connectTimer(this),
     msc(0), queuedMsc(0), frameTimer(this)
 {
-  slog.debug("creating single-threaded server %s", name.c_str());
+  slog.debug("Creating single-threaded server %s", name.c_str());
 
   desktop_->init(this);
 
   // FIXME: Do we really want to kick off these right away?
   if (rfb::Server::maxIdleTime)
-    idleTimer.start(secsToMillis(rfb::Server::maxIdleTime));
+    idleTimer.start(core::secsToMillis(rfb::Server::maxIdleTime));
   if (rfb::Server::maxDisconnectionTime)
-    disconnectTimer.start(secsToMillis(rfb::Server::maxDisconnectionTime));
+    disconnectTimer.start(core::secsToMillis(rfb::Server::maxDisconnectionTime));
 }
 
 VNCServerST::~VNCServerST()
 {
-  slog.debug("shutting down server %s", name.c_str());
+  slog.debug("Shutting down server %s", name.c_str());
 
   // Close any active clients, with appropriate logging & cleanup
   closeClients("Server shutdown");
@@ -140,7 +143,7 @@ void VNCServerST::addSocket(network::Socket* sock, bool outgoing, AccessRights a
   // *** do this in getSecurity instead?
   const char *address = sock->getPeerAddress();
   if (blHosts->isBlackmarked(address)) {
-    connectionsLog.error("blacklisted: %s", address);
+    connectionsLog.error("Blacklisted: %s", address);
     try {
       rdr::OutStream& os = sock->outStream();
 
@@ -151,23 +154,29 @@ void VNCServerST::addSocket(network::Socket* sock, bool outgoing, AccessRights a
       os.writeU32(strlen(reason));
       os.writeBytes((const uint8_t*)reason, strlen(reason));
       os.flush();
-    } catch (rdr::Exception&) {
+    } catch (std::exception&) {
     }
     sock->shutdown();
     closingSockets.push_back(sock);
     return;
   }
 
-  connectionsLog.status("accepted: %s", sock->getPeerEndpoint());
+  connectionsLog.status("Accepted: %s", sock->getPeerEndpoint());
 
   // Adjust the exit timers
   if (rfb::Server::maxConnectionTime && clients.empty())
-    connectTimer.start(secsToMillis(rfb::Server::maxConnectionTime));
+    connectTimer.start(core::secsToMillis(rfb::Server::maxConnectionTime));
   disconnectTimer.stop();
 
-  VNCSConnectionST* client = new VNCSConnectionST(this, sock, outgoing, accessRights);
-  clients.push_front(client);
-  client->init();
+  try {
+    VNCSConnectionST* client = new VNCSConnectionST(this, sock, outgoing, accessRights);
+    clients.push_front(client);
+    client->init();
+  } catch (std::exception& e) {
+    connectionsLog.error("Error accepting client: %s", e.what());
+    sock->shutdown();
+    closingSockets.push_back(sock);
+  }
 }
 
 void VNCServerST::removeSocket(network::Socket* sock) {
@@ -176,20 +185,23 @@ void VNCServerST::removeSocket(network::Socket* sock) {
   for (ci = clients.begin(); ci != clients.end(); ci++) {
     if ((*ci)->getSock() == sock) {
       // - Remove any references to it
-      if (pointerClient == *ci)
-        pointerClient = NULL;
+      if (pointerClient == *ci) {
+        // Release the mouse buttons the client have pressed
+        desktop->pointerEvent(cursorPos, 0);
+        pointerClient = nullptr;
+      }
       if (clipboardClient == *ci)
         handleClipboardAnnounce(*ci, false);
       clipboardRequestors.remove(*ci);
 
-      std::string name((*ci)->getPeerEndpoint());
+      std::string peer((*ci)->getPeerEndpoint());
 
       // - Delete the per-Socket resources
       delete *ci;
 
       clients.remove(*ci);
 
-      connectionsLog.status("closed: %s", name.c_str());
+      connectionsLog.status("Closed: %s", peer.c_str());
 
       // - Check that the desktop object is still required
       if (authClientCount() == 0)
@@ -201,7 +213,7 @@ void VNCServerST::removeSocket(network::Socket* sock) {
       // Adjust the exit timers
       connectTimer.stop();
       if (rfb::Server::maxDisconnectionTime && clients.empty())
-        disconnectTimer.start(secsToMillis(rfb::Server::maxDisconnectionTime));
+        disconnectTimer.start(core::secsToMillis(rfb::Server::maxDisconnectionTime));
 
       return;
     }
@@ -221,7 +233,7 @@ void VNCServerST::processSocketReadEvent(network::Socket* sock)
       return;
     }
   }
-  throw rdr::Exception("invalid Socket in VNCServerST");
+  throw std::invalid_argument("Invalid Socket in VNCServerST");
 }
 
 void VNCServerST::processSocketWriteEvent(network::Socket* sock)
@@ -234,7 +246,7 @@ void VNCServerST::processSocketWriteEvent(network::Socket* sock)
       return;
     }
   }
-  throw rdr::Exception("invalid Socket in VNCServerST");
+  throw std::invalid_argument("Invalid Socket in VNCServerST");
 }
 
 void VNCServerST::blockUpdates()
@@ -275,19 +287,19 @@ void VNCServerST::setPixelBuffer(PixelBuffer* pb_, const ScreenSet& layout)
 
   pb = pb_;
   delete comparer;
-  comparer = 0;
+  comparer = nullptr;
 
   if (!pb) {
     screenLayout = ScreenSet();
 
     if (desktopStarted)
-      throw Exception("setPixelBuffer: null PixelBuffer when desktopStarted?");
+      throw std::logic_error("setPixelBuffer: Null PixelBuffer when desktopStarted?");
 
     return;
   }
 
   if (!layout.validate(pb->width(), pb->height()))
-    throw Exception("setPixelBuffer: invalid screen layout");
+    throw std::invalid_argument("setPixelBuffer: Invalid screen layout");
 
   screenLayout = layout;
 
@@ -297,9 +309,11 @@ void VNCServerST::setPixelBuffer(PixelBuffer* pb_, const ScreenSet& layout)
   renderedCursorInvalid = true;
   add_changed(pb->getRect());
 
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci=clients.begin();ci!=clients.end();ci=ci_next) {
-    ci_next = ci; ci_next++;
+  // The desktop is considered ready after the pixelbuffer is set
+  checkDesktopReady();
+
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci) {
     (*ci)->pixelBufferChange();
     // Since the new pixel buffer means an ExtendedDesktopSize needs to
     // be sent anyway, we don't need to call screenLayoutChange.
@@ -310,9 +324,14 @@ void VNCServerST::setPixelBuffer(PixelBuffer* pb_)
 {
   ScreenSet layout = screenLayout;
 
+  if (!pb_) {
+    setPixelBuffer(nullptr, ScreenSet());
+    return;
+  }
+
   // Check that the screen layout is still valid
-  if (pb_ && !layout.validate(pb_->width(), pb_->height())) {
-    Rect fbRect;
+  if (!layout.validate(pb_->width(), pb_->height())) {
+    core::Rect fbRect;
     ScreenSet::iterator iter, iter_next;
 
     fbRect.setXYWH(0, 0, pb_->width(), pb_->height());
@@ -340,22 +359,23 @@ void VNCServerST::setPixelBuffer(PixelBuffer* pb_)
 void VNCServerST::setScreenLayout(const ScreenSet& layout)
 {
   if (!pb)
-    throw Exception("setScreenLayout: new screen layout without a PixelBuffer");
+    throw std::logic_error("setScreenLayout: New screen layout without a PixelBuffer");
   if (!layout.validate(pb->width(), pb->height()))
-    throw Exception("setScreenLayout: invalid screen layout");
+    throw std::invalid_argument("setScreenLayout: Invalid screen layout");
 
   screenLayout = layout;
 
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci=clients.begin();ci!=clients.end();ci=ci_next) {
-    ci_next = ci; ci_next++;
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci)
     (*ci)->screenLayoutChangeOrClose(reasonServer);
-  }
 }
 
 void VNCServerST::requestClipboard()
 {
-  if (clipboardClient == NULL) {
+  if (!rfb::Server::acceptCutText)
+    return;
+
+  if (clipboardClient == nullptr) {
     slog.debug("Got request for client clipboard but no client currently owns the clipboard");
     return;
   }
@@ -365,70 +385,105 @@ void VNCServerST::requestClipboard()
 
 void VNCServerST::announceClipboard(bool available)
 {
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
+  std::list<VNCSConnectionST*>::iterator ci;
 
   clipboardRequestors.clear();
 
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  if (!rfb::Server::sendCutText)
+    return;
+
+  for (ci = clients.begin(); ci != clients.end(); ++ci)
     (*ci)->announceClipboardOrClose(available);
-  }
 }
 
 void VNCServerST::sendClipboardData(const char* data)
 {
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
+  std::list<VNCSConnectionST*>::iterator ci;
 
-  if (strchr(data, '\r') != NULL)
-    throw Exception("Invalid carriage return in clipboard data");
+  if (!rfb::Server::sendCutText)
+    return;
+
+  if (strchr(data, '\r') != nullptr)
+    throw std::invalid_argument("Invalid carriage return in clipboard data");
 
   for (ci = clipboardRequestors.begin();
-       ci != clipboardRequestors.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+       ci != clipboardRequestors.end(); ++ci)
     (*ci)->sendClipboardDataOrClose(data);
-  }
 
   clipboardRequestors.clear();
 }
 
+void VNCServerST::checkDesktopReady()
+{
+  if (!desktopStarting)
+    return;
+
+  if (!pb)
+    return;
+
+  desktopStarting = false;
+  desktopReady();
+}
+
+void VNCServerST::desktopReady()
+{
+  if (!pb)
+      throw std::logic_error("desktopReady: Null PixelBuffer when desktop is ready");
+
+  desktopStarted = true;
+  // The tracker might have accumulated changes whilst we were
+  // stopped, so flush those out
+  assert(comparer != nullptr);
+  if (!comparer->is_empty())
+    writeUpdate();
+  // If the frame clock is running, then it will be running slowly,
+  // so give it a kick to run at normal speed right away
+  if (frameTimer.isStarted()) {
+    stopFrameClock();
+    startFrameClock();
+  }
+
+  // Now that the desktop is ready, client connections can continue
+  for (VNCSConnectionST* client : clients)
+    client->desktopReadyOrClose();
+}
+
 void VNCServerST::bell()
 {
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci)
     (*ci)->bellOrClose();
-  }
 }
 
 void VNCServerST::setName(const char* name_)
 {
   name = name_;
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci)
     (*ci)->setDesktopNameOrClose(name_);
-  }
 }
 
-void VNCServerST::add_changed(const Region& region)
+void VNCServerST::add_changed(const core::Region& region)
 {
-  if (comparer == NULL)
+  if (comparer == nullptr)
     return;
 
   comparer->add_changed(region);
   startFrameClock();
 }
 
-void VNCServerST::add_copied(const Region& dest, const Point& delta)
+void VNCServerST::add_copied(const core::Region& dest,
+                             const core::Point& delta)
 {
-  if (comparer == NULL)
+  if (comparer == nullptr)
     return;
 
   comparer->add_copied(dest, delta);
   startFrameClock();
 }
 
-void VNCServerST::setCursor(int width, int height, const Point& newHotspot,
+void VNCServerST::setCursor(int width, int height,
+                            const core::Point& newHotspot,
                             const uint8_t* data)
 {
   delete cursor;
@@ -437,15 +492,14 @@ void VNCServerST::setCursor(int width, int height, const Point& newHotspot,
 
   renderedCursorInvalid = true;
 
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci) {
     (*ci)->renderedCursorChange();
     (*ci)->setCursorOrClose();
   }
 }
 
-void VNCServerST::setCursorPos(const Point& pos, bool warped)
+void VNCServerST::setCursorPos(const core::Point& pos, bool warped)
 {
   if (cursorPos != pos) {
     cursorPos = pos;
@@ -461,25 +515,26 @@ void VNCServerST::setCursorPos(const Point& pos, bool warped)
 
 void VNCServerST::setLEDState(unsigned int state)
 {
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
+  std::list<VNCSConnectionST*>::iterator ci;
 
   if (state == ledState)
     return;
 
   ledState = state;
 
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  for (ci = clients.begin(); ci != clients.end(); ++ci)
     (*ci)->setLEDStateOrClose(state);
-  }
 }
 
 // Event handlers
 
 void VNCServerST::keyEvent(uint32_t keysym, uint32_t keycode, bool down)
 {
+  if (!rfb::Server::acceptKeyEvents)
+    return;
+
   if (rfb::Server::maxIdleTime)
-    idleTimer.start(secsToMillis(rfb::Server::maxIdleTime));
+    idleTimer.start(core::secsToMillis(rfb::Server::maxIdleTime));
 
   // Remap the key if required
   if (keyRemapper) {
@@ -496,16 +551,21 @@ void VNCServerST::keyEvent(uint32_t keysym, uint32_t keycode, bool down)
 }
 
 void VNCServerST::pointerEvent(VNCSConnectionST* client,
-                               const Point& pos, int buttonMask)
+                               const core::Point& pos,
+                               uint16_t buttonMask)
 {
-  time_t now = time(0);
+  time_t now = time(nullptr);
+
+  if (!rfb::Server::acceptPointerEvents)
+    return;
+
   if (rfb::Server::maxIdleTime)
-    idleTimer.start(secsToMillis(rfb::Server::maxIdleTime));
+    idleTimer.start(core::secsToMillis(rfb::Server::maxIdleTime));
 
   // Let one client own the cursor whilst buttons are pressed in order
   // to provide a bit more sane user experience. But limit the time to
   // prevent locking out all others when e.g. the network is down.
-  if ((pointerClient != NULL) && (pointerClient != client) &&
+  if ((pointerClient != nullptr) && (pointerClient != client) &&
       ((now - pointerClientTime) < 10))
     return;
 
@@ -513,7 +573,7 @@ void VNCServerST::pointerEvent(VNCSConnectionST* client,
   if (buttonMask)
     pointerClient = client;
   else
-    pointerClient = NULL;
+    pointerClient = nullptr;
 
   desktop->pointerEvent(pos, buttonMask);
 }
@@ -528,12 +588,14 @@ void VNCServerST::handleClipboardRequest(VNCSConnectionST* client)
 void VNCServerST::handleClipboardAnnounce(VNCSConnectionST* client,
                                           bool available)
 {
-  if (available)
+  if (available) {
+    if (!rfb::Server::acceptCutText)
+      return;
     clipboardClient = client;
-  else {
+  } else {
     if (client != clipboardClient)
       return;
-    clipboardClient = NULL;
+    clipboardClient = nullptr;
   }
   desktop->handleClipboardAnnounce(available);
 }
@@ -541,6 +603,8 @@ void VNCServerST::handleClipboardAnnounce(VNCSConnectionST* client,
 void VNCServerST::handleClipboardData(VNCSConnectionST* client,
                                       const char* data)
 {
+  if (!rfb::Server::acceptCutText)
+    return;
   if (client != clipboardClient) {
     slog.debug("Ignoring unexpected clipboard data");
     return;
@@ -553,7 +617,12 @@ unsigned int VNCServerST::setDesktopSize(VNCSConnectionST* requester,
                                          const ScreenSet& layout)
 {
   unsigned int result;
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
+  std::list<VNCSConnectionST*>::iterator ci;
+
+  if (!rfb::Server::acceptSetDesktopSize) {
+    slog.debug("Rejecting unauthorized framebuffer resize request");
+    return resultProhibited;
+  }
 
   // We can't handle a framebuffer larger than this, so don't let a
   // client set one (see PixelBuffer.cxx)
@@ -577,11 +646,10 @@ unsigned int VNCServerST::setDesktopSize(VNCSConnectionST* requester,
 
   // Sanity check
   if (screenLayout != layout)
-    throw Exception("Desktop configured a different screen layout than requested");
+    throw std::runtime_error("Desktop configured a different screen layout than requested");
 
   // Notify other clients
-  for (ci=clients.begin();ci!=clients.end();ci=ci_next) {
-    ci_next = ci; ci_next++;
+  for (ci = clients.begin(); ci != clients.end(); ++ci) {
     if ((*ci) == requester)
       continue;
     (*ci)->screenLayoutChangeOrClose(reasonOtherClient);
@@ -606,9 +674,8 @@ void VNCServerST::approveConnection(network::Socket* sock, bool accept,
 
 void VNCServerST::closeClients(const char* reason, network::Socket* except)
 {
-  std::list<VNCSConnectionST*>::iterator i, next_i;
-  for (i=clients.begin(); i!=clients.end(); i=next_i) {
-    next_i = i; next_i++;
+  std::list<VNCSConnectionST*>::iterator i;
+  for (i = clients.begin(); i != clients.end(); ++i) {
     if ((*i)->getSock() != except)
       (*i)->close(reason);
   }
@@ -633,10 +700,10 @@ SConnection* VNCServerST::getConnection(network::Socket* sock) {
     if ((*ci)->getSock() == sock)
       return (SConnection*)*ci;
   }
-  return 0;
+  return nullptr;
 }
 
-void VNCServerST::handleTimeout(Timer* t)
+void VNCServerST::handleTimeout(core::Timer* t)
 {
   if (t == &frameTimer) {
     int timeout;
@@ -644,7 +711,7 @@ void VNCServerST::handleTimeout(Timer* t)
     // We keep running until we go a full interval without any updates,
     // or there are no active clients anymore
     if (!desktopStarted ||
-        ((comparer != NULL) && comparer->is_empty())) {
+        ((comparer != nullptr) && comparer->is_empty())) {
       // Unless something waits for us to advance the frame count
       if (queuedMsc < msc)
         return;
@@ -660,7 +727,7 @@ void VNCServerST::handleTimeout(Timer* t)
     frameTimer.repeat(timeout);
 
     if (desktopStarted &&
-        ((comparer != NULL) && !comparer->is_empty()))
+        ((comparer != nullptr) && !comparer->is_empty()))
       writeUpdate();
 
     msc++;
@@ -699,14 +766,14 @@ void VNCServerST::queryConnection(VNCSConnectionST* client,
   // - Are we configured to do queries?
   if (!rfb::Server::queryConnect &&
       !client->getSock()->requiresQuery()) {
-    approveConnection(client->getSock(), true, NULL);
+    approveConnection(client->getSock(), true, nullptr);
     return;
   }
 
   // - Does the client have the right to bypass the query?
   if (client->accessCheck(AccessNoQuery))
   {
-    approveConnection(client->getSock(), true, NULL);
+    approveConnection(client->getSock(), true, nullptr);
     return;
   }
 
@@ -719,7 +786,7 @@ void VNCServerST::clientReady(VNCSConnectionST* client, bool shared)
     if (rfb::Server::disconnectClients &&
         client->accessCheck(AccessNonShared)) {
       // - Close all the other connected clients
-      slog.debug("non-shared connection - closing clients");
+      slog.debug("Non-shared connection - closing clients");
       closeClients("Non-shared connection requested", client->getSock());
     } else {
       // - Refuse this connection if there are existing clients, in addition to
@@ -736,31 +803,21 @@ void VNCServerST::clientReady(VNCSConnectionST* client, bool shared)
 
 void VNCServerST::startDesktop()
 {
-  if (!desktopStarted) {
-    slog.debug("starting desktop");
+  if (!desktopStarted && !desktopStarting) {
+    slog.debug("Starting desktop");
+    desktopStarting = true;
     desktop->start();
-    if (!pb)
-      throw Exception("SDesktop::start() did not set a valid PixelBuffer");
-    desktopStarted = true;
-    // The tracker might have accumulated changes whilst we were
-    // stopped, so flush those out
-    assert(comparer != NULL);
-    if (!comparer->is_empty())
-      writeUpdate();
-    // If the frame clock is running, then it will be running slowly,
-    // so give it a kick to run at normal speed right away
-    if (frameTimer.isStarted()) {
-      stopFrameClock();
-      startFrameClock();
-    }
+    // We might have already been in a ready state
+    checkDesktopReady();
   }
 }
 
 void VNCServerST::stopDesktop()
 {
-  if (desktopStarted) {
-    slog.debug("stopping desktop");
+  if (desktopStarted || desktopStarting) {
+    slog.debug("Stopping desktop");
     desktopStarted = false;
+    desktopStarting = false;
     desktop->stop();
   }
 }
@@ -792,7 +849,7 @@ void VNCServerST::startFrameClock()
 
   // Anyone actually interested in frames?
   if (!desktopStarted ||
-      ((comparer != NULL) && comparer->is_empty())) {
+      ((comparer != nullptr) && comparer->is_empty())) {
     if (queuedMsc < msc)
       return;
   }
@@ -836,21 +893,21 @@ int VNCServerST::msToNextUpdate()
 void VNCServerST::writeUpdate()
 {
   UpdateInfo ui;
-  Region toCheck;
+  core::Region toCheck;
 
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
+  std::list<VNCSConnectionST*>::iterator ci;
 
   assert(blockCounter == 0);
   assert(desktopStarted);
-  assert(comparer != NULL);
+  assert(comparer != nullptr);
 
   comparer->getUpdateInfo(&ui, pb->getRect());
   toCheck = ui.changed.union_(ui.copied);
 
   if (needRenderedCursor()) {
-    Rect clippedCursorRect = Rect(0, 0, cursor->width(), cursor->height())
-                             .translate(cursorPos.subtract(cursor->hotspot()))
-                             .intersect(pb->getRect());
+    core::Rect clippedCursorRect = core::Rect(0, 0, cursor->width(), cursor->height())
+                                   .translate(cursorPos.subtract(cursor->hotspot()))
+                                   .intersect(pb->getRect());
 
     if (!toCheck.intersect(clippedCursorRect).is_empty())
       renderedCursorInvalid = true;
@@ -868,8 +925,7 @@ void VNCServerST::writeUpdate()
 
   comparer->clear();
 
-  for (ci = clients.begin(); ci != clients.end(); ci = ci_next) {
-    ci_next = ci; ci_next++;
+  for (ci = clients.begin(); ci != clients.end(); ++ci) {
     (*ci)->add_copied(ui.copied, ui.copy_delta);
     (*ci)->add_changed(ui.changed);
     (*ci)->writeFramebufferUpdateOrClose();
@@ -879,7 +935,7 @@ void VNCServerST::writeUpdate()
 // checkUpdate() is called by clients to see if it is safe to read from
 // the framebuffer at this time.
 
-Region VNCServerST::getPendingRegion()
+core::Region VNCServerST::getPendingRegion()
 {
   UpdateInfo ui;
 
@@ -887,11 +943,11 @@ Region VNCServerST::getPendingRegion()
   if (blockCounter > 0)
     return pb->getRect();
 
-  assert(comparer != NULL);
+  assert(comparer != nullptr);
 
   // Block client from updating if there are pending updates
   if (comparer->is_empty())
-    return Region();
+    return {};
 
   comparer->getUpdateInfo(&ui, pb->getRect());
 
@@ -915,9 +971,8 @@ bool VNCServerST::getComparerState()
   if (rfb::Server::compareFB != 2)
     return true;
 
-  std::list<VNCSConnectionST*>::iterator ci, ci_next;
-  for (ci=clients.begin();ci!=clients.end();ci=ci_next) {
-    ci_next = ci; ci_next++;
+  std::list<VNCSConnectionST*>::iterator ci;
+  for (ci = clients.begin(); ci != clients.end(); ++ci) {
     if ((*ci)->getComparerState())
       return true;
   }
