@@ -135,6 +135,7 @@ class OptionsDialog extends Dialog {
   JCheckBox viewOnlyCheckbox;
   JCheckBox acceptClipboardCheckbox;
   JCheckBox sendClipboardCheckbox;
+  JCheckBox disableArrowScrollCheckbox;
   JComboBox menuKeyChoice;
 
   /* Screen */
@@ -152,9 +153,11 @@ class OptionsDialog extends Dialog {
 
   /* Misc. */
   JCheckBox sharedCheckbox;
-  JCheckBox dotWhenNoCursorCheckbox;
+  JCheckBox alwaysCursorCheckbox;
   JCheckBox acceptBellCheckbox;
 
+  JComboBox cursorTypeChoice;
+
   /* SSH */
   JCheckBox tunnelCheckbox;
   JCheckBox viaCheckbox;
@@ -175,7 +178,7 @@ class OptionsDialog extends Dialog {
   @SuppressWarnings({"rawtypes","unchecked"})
   public OptionsDialog() {
     super(true);
-    setTitle("VNC Viewer Options");
+    setTitle("TigerVNC options");
     setResizable(false);
 
     getContentPane().setLayout(
@@ -311,6 +314,7 @@ class OptionsDialog extends Dialog {
     handleAutoselect();
     handleCompression();
     handleJpeg();
+    handleAlwaysCursor();
 
     /* Security */
     Security security = new Security(SecurityClient.secTypes);
@@ -414,6 +418,7 @@ class OptionsDialog extends Dialog {
     viewOnlyCheckbox.setSelected(viewOnly.getValue());
     acceptClipboardCheckbox.setSelected(acceptClipboard.getValue());
     sendClipboardCheckbox.setSelected(sendClipboard.getValue());
+    disableArrowScrollCheckbox.setSelected(disableArrowScroll.getValue());
 
     menuKeyChoice.setSelectedIndex(0);
 
@@ -458,7 +463,9 @@ class OptionsDialog extends Dialog {
 
     /* Misc. */
     sharedCheckbox.setSelected(shared.getValue());
-    dotWhenNoCursorCheckbox.setSelected(dotWhenNoCursor.getValue());
+    alwaysCursorCheckbox.setSelected(alwaysCursor.getValue());
+    String cursorTypeStr = cursorType.getValueStr();
+    cursorTypeChoice.setSelectedItem(cursorTypeStr);
     acceptBellCheckbox.setSelected(acceptBell.getValue());
 
     /* SSH */
@@ -587,6 +594,7 @@ class OptionsDialog extends Dialog {
     viewOnly.setParam(viewOnlyCheckbox.isSelected());
     acceptClipboard.setParam(acceptClipboardCheckbox.isSelected());
     sendClipboard.setParam(sendClipboardCheckbox.isSelected());
+    disableArrowScroll.setParam(disableArrowScrollCheckbox.isSelected());
 
     String menuKeyStr =
       MenuKey.getMenuKeySymbols()[menuKeyChoice.getSelectedIndex()].name;
@@ -608,13 +616,14 @@ class OptionsDialog extends Dialog {
 
     String scaleStr =
       ((String)scalingFactorInput.getSelectedItem()).replace("%", "");
-    scaleStr.replace("Fixed Aspect Ratio", "FixedRatio");
+    scaleStr = scaleStr.replace("Fixed Aspect Ratio", "FixedRatio");
     scalingFactor.setParam(scaleStr);
 
     /* Misc. */
     shared.setParam(sharedCheckbox.isSelected());
-    dotWhenNoCursor.setParam(dotWhenNoCursorCheckbox.isSelected());
+    alwaysCursor.setParam(alwaysCursorCheckbox.isSelected());
     acceptBell.setParam(acceptBellCheckbox.isSelected());
+    cursorType.setParam((String)cursorTypeChoice.getSelectedItem());
 
     /* SSH */
     tunnel.setParam(tunnelCheckbox.isSelected());
@@ -1000,6 +1009,7 @@ class OptionsDialog extends Dialog {
     viewOnlyCheckbox = new JCheckBox("View only (ignore mouse and keyboard)");
     acceptClipboardCheckbox = new JCheckBox("Accept clipboard from server");
     sendClipboardCheckbox = new JCheckBox("Send clipboard to server");
+    disableArrowScrollCheckbox = new JCheckBox("Disable arrow-key scrolling when scrollbars visible");
     JLabel menuKeyLabel = new JLabel("Menu key");
     String[] menuKeys = new String[MenuKey.getMenuKeySymbolCount()];
     //menuKeys[0] = "None";
@@ -1028,15 +1038,22 @@ class OptionsDialog extends Dialog {
                                           LINE_START, NONE,
                                           new Insets(0, 0, 4, 0),
                                           NONE, NONE));
-    inputPanel.add(menuKeyLabel,
+    inputPanel.add(disableArrowScrollCheckbox,
                    new GridBagConstraints(0, 3,
+                                          REMAINDER, 1,
+                                          HEAVY, LIGHT,
+                                          LINE_START, NONE,
+                                          new Insets(0, 0, 4, 0),
+                                          NONE, NONE));
+    inputPanel.add(menuKeyLabel,
+                   new GridBagConstraints(0, 4,
                                           1, 1,
                                           LIGHT, LIGHT,
                                           LINE_START, NONE,
                                           new Insets(0, 0, 0, 0),
                                           NONE, NONE));
     inputPanel.add(menuKeyChoice,
-                   new GridBagConstraints(1, 3,
+                   new GridBagConstraints(1, 4,
                                           1, 1,
                                           HEAVY, LIGHT,
                                           LINE_START, NONE,
@@ -1173,31 +1190,54 @@ class OptionsDialog extends Dialog {
     MiscPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
     sharedCheckbox =
       new JCheckBox("Shared (don't disconnect other viewers)");
-    dotWhenNoCursorCheckbox = new JCheckBox("Show dot when no cursor");
+    alwaysCursorCheckbox = new JCheckBox("Show local cursor when not provided by server");
+    alwaysCursorCheckbox.addItemListener(new ItemListener() {
+      public void itemStateChanged(ItemEvent e) {
+        handleAlwaysCursor();
+      }
+    });
+    JLabel cursorTypeLabel = new JLabel("Cursor type:");
+    String[] cursorTypes = {"Dot", "System"};
+    cursorTypeChoice = new MyJComboBox(cursorTypes);
+    cursorTypeChoice.setPrototypeDisplayValue("System.");
     acceptBellCheckbox = new JCheckBox("Beep when requested by the server");
     MiscPanel.add(sharedCheckbox,
                   new GridBagConstraints(0, 0,
-                                         1, 1,
+                                         REMAINDER, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(dotWhenNoCursorCheckbox,
+    MiscPanel.add(alwaysCursorCheckbox,
                   new GridBagConstraints(0, 1,
-                                         1, 1,
+                                         REMAINDER, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(acceptBellCheckbox,
+    MiscPanel.add(cursorTypeLabel,
                   new GridBagConstraints(0, 2,
                                          1, 1,
                                          LIGHT, LIGHT,
                                          LINE_START, NONE,
                                          new Insets(0, 0, 4, 0),
                                          NONE, NONE));
-    MiscPanel.add(Box.createRigidArea(new Dimension(5, 0)),
+    MiscPanel.add(cursorTypeChoice,
+                  new GridBagConstraints(1, 2,
+                                         1, 1,
+                                         LIGHT, LIGHT,
+                                         LINE_START, NONE,
+                                         new Insets(0, 5, 4, 0),
+                                         NONE, NONE));
+    MiscPanel.add(acceptBellCheckbox,
                   new GridBagConstraints(0, 3,
+                                         REMAINDER, 1,
+                                         LIGHT, LIGHT,
+                                         LINE_START, NONE,
+                                         new Insets(0, 0, 4, 0),
+                                         NONE, NONE));
+    MiscPanel.add(Box.createRigidArea(new Dimension(5, 0)),
+                  new GridBagConstraints(0, 4,
                                          REMAINDER, REMAINDER,
                                          HEAVY, HEAVY,
                                          LINE_START, BOTH,
@@ -1633,5 +1673,10 @@ class OptionsDialog extends Dialog {
     }
   }
 
+  private void handleAlwaysCursor()
+  {
+    cursorTypeChoice.setEnabled(alwaysCursorCheckbox.isSelected());
+  }
+
   static LogWriter vlog = new LogWriter("OptionsDialog");
 }
