@@ -56,7 +56,8 @@ class Viewport extends JPanel implements ActionListener {
 
   enum ID { EXIT, FULLSCREEN, MINIMIZE, RESIZE, NEWVIEWER,
             CTRL, ALT, MENUKEY, CTRLALTDEL, CLIPBOARD,
-            REFRESH, OPTIONS, INFO, ABOUT, DISMISS }
+            REFRESH, OPTIONS, INFO, ABOUT, DISMISS,
+            ZOOM_IN, ZOOM_OUT, ZOOM_RESET, ZOOM_TO_FIT }
 
   enum MENU { INACTIVE, TOGGLE, VALUE, RADIO,
               INVISIBLE, SUBMENU_POINTER, SUBMENU, DIVIDER }
@@ -149,11 +150,11 @@ class Viewport extends JPanel implements ActionListener {
   }
 
   static final int[] dotcursor_xpm = {
-    0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
-    0xffffffff, 0xff000000, 0xff000000, 0xff000000, 0xffffffff,
-    0xffffffff, 0xff000000, 0xff000000, 0xff000000, 0xffffffff,
-    0xffffffff, 0xff000000, 0xff000000, 0xff000000, 0xffffffff,
-    0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
+    0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
+    0x00000000, 0xff000000, 0xff000000, 0xff000000, 0x00000000,
+    0x00000000, 0xff000000, 0xff000000, 0xff000000, 0x00000000,
+    0x00000000, 0xff000000, 0xff000000, 0xff000000, 0x00000000,
+    0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
   };
 
   public void setCursor(int width, int height, Point hotspot,
@@ -167,8 +168,15 @@ class Viewport extends JPanel implements ActionListener {
     for (i = 0; i < width*height; i++)
       if (data[i*4 + 3] != 0) break;
 
-    if ((i == width*height) && dotWhenNoCursor.getValue()) {
-      vlog.debug("cursor is empty - using dot");
+    useSystemCursor = false;
+    if ((i == width*height) && alwaysCursor.getValue()) {
+      String cursorTypeStr = cursorType.getValueStr();
+      if (cursorTypeStr.matches("^System$")) {
+        useSystemCursor = true;
+      } else {
+        vlog.debug("Cursor is empty: Using dot");
+      }
+      // Do this anyway to prevent cursor being null
       cursor = new BufferedImage(5, 5, BufferedImage.TYPE_INT_ARGB_PRE);
       cursor.setRGB(0, 0, 5, 5, dotcursor_xpm, 0, 5);
       cursorHotspot.x = cursorHotspot.y = 3;
@@ -188,23 +196,20 @@ class Viewport extends JPanel implements ActionListener {
       }
     }
 
-    int cw = (int)Math.floor((float)cursor.getWidth() * scaleRatioX);
-    int ch = (int)Math.floor((float)cursor.getHeight() * scaleRatioY);
-
-    int x = (int)Math.floor((float)cursorHotspot.x * scaleRatioX);
-    int y = (int)Math.floor((float)cursorHotspot.y * scaleRatioY);
-
+    int cw = (int) Math.floor((float) cursor.getWidth() * scaleRatioX);
+    int ch = (int) Math.floor((float) cursor.getHeight() * scaleRatioY);
+    int x = cursorHotspot.x;
+    int y = cursorHotspot.y;
     Dimension cs = tk.getBestCursorSize(cw, ch);
-    if (cs.width != cw && cs.height != ch) {
-      cw = Math.min(cw, cs.width);
-      ch = Math.min(ch, cs.height);
-      x = (int)Math.min(x, Math.max(cs.width - 1, 0));
-      y = (int)Math.min(y, Math.max(cs.height - 1, 0));
-      BufferedImage tmp =
-        new BufferedImage(cs.width, cs.height, BufferedImage.TYPE_INT_ARGB_PRE);
+    if (cs.width != cursor.getWidth() || cs.height != cursor.getHeight()) {
+      cw = VncViewer.os.startsWith("windows") ?  Math.min(cw, cs.width) : cs.width;
+      ch = VncViewer.os.startsWith("windows") ?  Math.min(ch, cs.height) : cs.height;
+      BufferedImage tmp = new BufferedImage(cs.width, cs.height, BufferedImage.TYPE_INT_ARGB_PRE);
       Graphics2D g2 = tmp.createGraphics();
       g2.drawImage(cursor, 0, 0, cw, ch, 0, 0, width, height, null);
       g2.dispose();
+      x = (int) Math.min(Math.floor((float) x * (float) cw / (float) width), Math.max(cw - 1, 0));
+      y = (int) Math.min(Math.floor((float) y * (float) ch / (float) height), Math.max(ch - 1, 0));
       cursor = tmp;
     }
 
@@ -219,7 +224,10 @@ class Viewport extends JPanel implements ActionListener {
 
     hotspot = new java.awt.Point(x, y);
     softCursor = tk.createCustomCursor(img, hotspot, name);
-    setCursor(softCursor);
+    if (useSystemCursor)
+      setCursor(java.awt.Cursor.getDefaultCursor());
+    else
+      setCursor(softCursor);
   }
 
   public void resize(int x, int y, int w, int h) {
@@ -640,9 +648,18 @@ class Viewport extends JPanel implements ActionListener {
              this, ID.OPTIONS, EnumSet.noneOf(MENU.class));
     menu_add(contextMenu, "Connection info...", KeyEvent.VK_I,
              this, ID.INFO, EnumSet.noneOf(MENU.class));
-    menu_add(contextMenu, "About TigerVNC viewer...", KeyEvent.VK_T,
+    menu_add(contextMenu, "About TigerVNC...", KeyEvent.VK_T,
              this, ID.ABOUT, EnumSet.of(MENU.DIVIDER));
 
+    menu_add(contextMenu, "Zoom in", KeyEvent.VK_EQUALS,
+             this, ID.ZOOM_IN, EnumSet.noneOf(MENU.class));
+    menu_add(contextMenu, "Zoom out", KeyEvent.VK_MINUS,
+             this, ID.ZOOM_OUT, EnumSet.noneOf(MENU.class));
+    menu_add(contextMenu, "Reset zoom", KeyEvent.VK_0,
+             this, ID.ZOOM_RESET, EnumSet.noneOf(MENU.class));
+    menu_add(contextMenu, "Zoom to fit (fixed aspect)", KeyEvent.VK_F,
+             this, ID.ZOOM_TO_FIT, EnumSet.of(MENU.DIVIDER));
+
     menu_add(contextMenu, "Dismiss menu", KeyEvent.VK_M,
              this, ID.DISMISS, EnumSet.noneOf(MENU.class));
   }
@@ -761,6 +778,18 @@ class Viewport extends JPanel implements ActionListener {
     case ABOUT:
       VncViewer.about_vncviewer(cc.desktop);
       break;
+    case ZOOM_IN:
+      if (!remoteResize.getValue()) window().adjustRelativeScalePercent(5);
+      break;
+    case ZOOM_OUT:
+      if (!remoteResize.getValue()) window().adjustRelativeScalePercent(-5);
+      break;
+    case ZOOM_RESET:
+      if (!remoteResize.getValue()) window().resetZoomToDefault();
+      break;
+    case ZOOM_TO_FIT:
+      if (!remoteResize.getValue()) window().setZoomToFit();
+      break;
     case DISMISS:
       break;
     }
@@ -833,6 +862,7 @@ class Viewport extends JPanel implements ActionListener {
   float scaleRatioX, scaleRatioY;
 
   static BufferedImage cursor;
+  static boolean useSystemCursor = false;
   Point cursorHotspot = new Point();
 
 }
