@@ -32,7 +32,8 @@
 #include <sys/ipc.h>
 #include <sys/shm.h>
 
-#include <rfb/LogWriter.h>
+#include <core/LogWriter.h>
+
 #include <x0vncserver/Image.h>
 
 //
@@ -60,16 +61,16 @@ ImageCleanup imageCleanup;
 // Image class implementation.
 //
 
-static rfb::LogWriter vlog("Image");
+static core::LogWriter vlog("Image");
 
 Image::Image(Display *d)
-  : xim(NULL), dpy(d)
+  : xim(nullptr), dpy(d)
 {
   imageCleanup.images.push_back(this);
 }
 
 Image::Image(Display *d, int width, int height)
-  : xim(NULL), dpy(d)
+  : xim(nullptr), dpy(d)
 {
   imageCleanup.images.push_back(this);
   Init(width, height);
@@ -80,15 +81,16 @@ void Image::Init(int width, int height)
   Visual* vis = DefaultVisual(dpy, DefaultScreen(dpy));
 
   if (vis->c_class != TrueColor) {
-    vlog.error("pseudocolour not supported");
+    vlog.error("Pseudocolour not supported");
     exit(1);
   }
 
   xim = XCreateImage(dpy, vis, DefaultDepth(dpy, DefaultScreen(dpy)),
-                     ZPixmap, 0, 0, width, height, BitmapPad(dpy), 0);
+                     ZPixmap, 0, nullptr, width, height,
+                     BitmapPad(dpy), 0);
 
   xim->data = (char *)malloc(xim->bytes_per_line * xim->height);
-  if (xim->data == NULL) {
+  if (xim->data == nullptr) {
     vlog.error("malloc() failed");
     exit(1);
   }
@@ -99,7 +101,7 @@ Image::~Image()
   imageCleanup.images.remove(this);
 
   // XDestroyImage will free xim->data if necessary
-  if (xim != NULL)
+  if (xim != nullptr)
     XDestroyImage(xim);
 }
 
@@ -217,12 +219,12 @@ static int ShmCreationXErrorHandler(Display* /*dpy*/,
 }
 
 ShmImage::ShmImage(Display *d)
-  : Image(d), shminfo(NULL)
+  : Image(d), shminfo(nullptr)
 {
 }
 
 ShmImage::ShmImage(Display *d, int width, int height)
-  : Image(d), shminfo(NULL)
+  : Image(d), shminfo(nullptr)
 {
   Init(width, height);
 }
@@ -241,7 +243,7 @@ void ShmImage::Init(int width, int height, const XVisualInfo *vinfo)
   Visual *visual;
   int depth;
 
-  if (vinfo == NULL) {
+  if (vinfo == nullptr) {
     visual = DefaultVisual(dpy, DefaultScreen(dpy));
     depth = DefaultDepth(dpy, DefaultScreen(dpy));
   } else {
@@ -250,18 +252,18 @@ void ShmImage::Init(int width, int height, const XVisualInfo *vinfo)
   }
 
   if (visual->c_class != TrueColor) {
-    vlog.error("pseudocolour not supported");
+    vlog.error("Pseudocolour not supported");
     exit(1);
   }
 
   shminfo = new XShmSegmentInfo;
 
-  xim = XShmCreateImage(dpy, visual, depth, ZPixmap, 0, shminfo,
+  xim = XShmCreateImage(dpy, visual, depth, ZPixmap, nullptr, shminfo,
 			width, height);
-  if (xim == NULL) {
+  if (xim == nullptr) {
     vlog.error("XShmCreateImage() failed");
     delete shminfo;
-    shminfo = NULL;
+    shminfo = nullptr;
     return;
   }
 
@@ -273,22 +275,22 @@ void ShmImage::Init(int width, int height, const XVisualInfo *vinfo)
     vlog.error("shmget() failed (%d bytes requested)",
                int(xim->bytes_per_line * xim->height));
     XDestroyImage(xim);
-    xim = NULL;
+    xim = nullptr;
     delete shminfo;
-    shminfo = NULL;
+    shminfo = nullptr;
     return;
   }
 
-  shminfo->shmaddr = xim->data = (char *)shmat(shminfo->shmid, 0, 0);
+  shminfo->shmaddr = xim->data = (char *)shmat(shminfo->shmid, nullptr, 0);
   if (shminfo->shmaddr == (char *)-1) {
     perror("shmat");
     vlog.error("shmat() failed (%d bytes requested)",
                int(xim->bytes_per_line * xim->height));
-    shmctl(shminfo->shmid, IPC_RMID, 0);
+    shmctl(shminfo->shmid, IPC_RMID, nullptr);
     XDestroyImage(xim);
-    xim = NULL;
+    xim = nullptr;
     delete shminfo;
-    shminfo = NULL;
+    shminfo = nullptr;
     return;
   }
 
@@ -301,21 +303,21 @@ void ShmImage::Init(int width, int height, const XVisualInfo *vinfo)
   if (caughtShmError) {
     vlog.error("XShmAttach() failed");
     shmdt(shminfo->shmaddr);
-    shmctl(shminfo->shmid, IPC_RMID, 0);
+    shmctl(shminfo->shmid, IPC_RMID, nullptr);
     XDestroyImage(xim);
-    xim = NULL;
+    xim = nullptr;
     delete shminfo;
-    shminfo = NULL;
+    shminfo = nullptr;
     return;
   }
 }
 
 ShmImage::~ShmImage()
 {
-  if (shminfo != NULL) {
+  if (shminfo != nullptr) {
     XShmDetach(dpy, shminfo);
     shmdt(shminfo->shmaddr);
-    shmctl(shminfo->shmid, IPC_RMID, 0);
+    shmctl(shminfo->shmid, IPC_RMID, nullptr);
     delete shminfo;
   }
 }
@@ -354,13 +356,13 @@ ImageFactory::~ImageFactory()
 
 Image *ImageFactory::newImage(Display *d, int width, int height)
 {
-  Image *image = NULL;
+  Image *image = nullptr;
 
   // Now, try to use shared memory image.
 
   if (mayUseShm) {
     image = new ShmImage(d, width, height);
-    if (image->xim != NULL) {
+    if (image->xim != nullptr) {
       return image;
     }
 
