@@ -41,6 +41,9 @@ from the X Consortium.
 #include "xorg-version.h"
 
 #include <stdio.h>
+#ifdef HAVE_LIBXCVT
+#include <libxcvt/libxcvt.h>
+#endif
 #include <X11/X.h>
 #include <X11/Xproto.h>
 #include <X11/Xos.h>
@@ -70,8 +73,8 @@ extern char buildtime[];
 
 #include "version-config.h"
 
-#define XVNCVERSION "TigerVNC 1.14.0"
-#define XVNCCOPYRIGHT ("Copyright (C) 1999-2024 TigerVNC Team and many others (see README.rst)\n" \
+#define XVNCVERSION "TigerVNC 1.15.90"
+#define XVNCCOPYRIGHT ("Copyright (C) 1999-2025 TigerVNC team and many others (see README.rst)\n" \
                        "See https://www.tigervnc.org for information on TigerVNC.\n")
 
 #define VNC_DEFAULT_WIDTH  1024
@@ -107,7 +110,6 @@ static VncScreenInfo vncScreenInfo = {
 static Bool vncPixmapDepths[33];
 static Bool Render = TRUE;
 
-static Bool displaySpecified = FALSE;
 static char displayNumStr[16];
 
 static int vncVerbose = 0;
@@ -173,15 +175,20 @@ ddxGiveUp(enum ExitCode error)
     vncFreeFramebufferMemory(&vncScreenInfo.fb);
 }
 
+#if XORG_OLDER_THAN(1, 21, 1)
 void
 AbortDDX(enum ExitCode error)
 {
     ddxGiveUp(error);
 }
+#endif
 
 void
 OsVendorInit(void)
 {
+    /* At this point, display has been set, so we can use it to
+     * initialize UnixPasswordValidator */
+    vncSetDisplayName(display);
 }
 
 void
@@ -211,8 +218,6 @@ ddxInputThreadInit(void)
 void
 ddxUseMsg(void)
 {
-    vncPrintBanner();
-
     ErrorF("-pixdepths list-of-int support given pixmap depths\n");
     ErrorF("+/-render		   turn on/off RENDER extension support"
            "(default on)\n");
@@ -275,14 +280,16 @@ ddxProcessArgument(int argc, char *argv[], int i)
     }
 
     if (argv[i][0] == ':')
-        displaySpecified = TRUE;
+        return 0;
 
+#if XORG_OLDER_THAN(1, 21, 1)
 #define CHECK_FOR_REQUIRED_ARGUMENTS(num) \
     if (((i + num) >= argc) || (!argv[i + num])) {                      \
       ErrorF("Required argument to %s not specified\n", argv[i]);       \
       UseMsg();                                                         \
       FatalError("Required argument to %s not specified\n", argv[i]);   \
     }
+#endif
 
     if (strcmp(argv[i], "-pixdepths") == 0) {   /* -pixdepths list-of-depth */
         int depth, ret = 1;
@@ -371,7 +378,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
 
         if ((vncInetdSock = dup(0)) == -1)
             FatalError
-                ("Xvnc error: failed to allocate a new file descriptor for -inetd: %s\n", strerror(errno));
+                ("Xvnc error: Failed to allocate a new file descriptor for -inetd: %s\n", strerror(errno));
 
 
         /* Avoid xserver >= 1.19's epoll-fd becoming fd 2 / stderr only to be
@@ -381,7 +388,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
         dup2(nullfd, 2);
         close(nullfd);
 
-        if (!displaySpecified) {
+        if (!explicit_display) {
             int port = vncGetSocketPort(vncInetdSock);
             int displayNum = port - 5900;
 
@@ -393,11 +400,11 @@ ddxProcessArgument(int argc, char *argv[], int i)
 
                 if (displayNum == 100)
                     FatalError
-                        ("Xvnc error: no free display number for -inetd\n");
+                        ("Xvnc error: No free display number for -inetd\n");
             }
-
-            display = displayNumStr;
             sprintf(displayNumStr, "%d", displayNum);
+            display = displayNumStr;
+            explicit_display = TRUE;
         }
 
         return 1;
@@ -445,28 +452,7 @@ ddxProcessArgument(int argc, char *argv[], int i)
         exit(0);
     }
 
-    /* We need to resolve an ambiguity for booleans */
-    if (argv[i][0] == '-' && i + 1 < argc && vncIsParamBool(&argv[i][1])) {
-        if ((strcasecmp(argv[i + 1], "0") == 0) ||
-            (strcasecmp(argv[i + 1], "1") == 0) ||
-            (strcasecmp(argv[i + 1], "true") == 0) ||
-            (strcasecmp(argv[i + 1], "false") == 0) ||
-            (strcasecmp(argv[i + 1], "yes") == 0) ||
-            (strcasecmp(argv[i + 1], "no") == 0)) {
-            vncSetParam(&argv[i][1], argv[i + 1]);
-            return 2;
-        }
-    }
-
-    if (vncSetParamSimple(argv[i]))
-        return 1;
-
-    if (argv[i][0] == '-' && i + 1 < argc) {
-        if (vncSetParam(&argv[i][1], argv[i + 1]))
-            return 2;
-    }
-
-    return 0;
+    return vncHandleParamArg(argc, argv, i);
 }
 
 static Bool
@@ -762,15 +748,40 @@ vncRandRModeGet(int width, int height)
     xRRModeInfo modeInfo;
     char name[100];
     RRModePtr mode;
+#ifdef HAVE_LIBXCVT
+    struct libxcvt_mode_info *cvtMode;
+#endif
 
     memset(&modeInfo, 0, sizeof(modeInfo));
-    sprintf(name, "%dx%d", width, height);
 
+#ifdef HAVE_LIBXCVT
+    cvtMode = libxcvt_gen_mode_info(width, height, 60.0, false, false);
+
+    modeInfo.width      = cvtMode->hdisplay;
+    modeInfo.height     = cvtMode->vdisplay;
+    modeInfo.dotClock   = cvtMode->dot_clock * 1000.0;
+    modeInfo.hSyncStart = cvtMode->hsync_start;
+    modeInfo.hSyncEnd   = cvtMode->hsync_end;
+    modeInfo.hTotal     = cvtMode->htotal;
+    modeInfo.vSyncStart = cvtMode->vsync_start;
+    modeInfo.vSyncEnd   = cvtMode->vsync_end;
+    modeInfo.vTotal     = cvtMode->vtotal;
+    modeInfo.modeFlags  = cvtMode->mode_flags;
+
+    free(cvtMode);
+
+    /* libxcvt rounds up to multiples of 8, so override them here */
+    modeInfo.width = width;
+    modeInfo.height = height;
+#else
     modeInfo.width = width;
     modeInfo.height = height;
     modeInfo.hTotal = width;
     modeInfo.vTotal = height;
     modeInfo.dotClock = ((CARD32) width * (CARD32) height * 60);
+#endif
+
+    sprintf(name, "%dx%d", width, height);
     modeInfo.nameLength = strlen(name);
     mode = RRModeGet(&modeInfo, name);
     if (mode == NULL)
@@ -876,7 +887,7 @@ vncRandRCreateScreenOutputs(int scrIdx, int extraOutputs)
 /* Creating and modifying modes, used by XserverDesktop and init here */
 
 int
-vncRandRCanCreateModes()
+vncRandRCanCreateModes(void)
 {
     return 1;
 }
@@ -1143,10 +1154,11 @@ InitOutput(ScreenInfo * scrInfo, int argc, char **argv)
     int i;
     int NumFormats = 0;
 
-    vncPrintBanner();
+    if (serverGeneration == 1) {
+        vncPrintBanner();
 
-    if (serverGeneration == 1)
         LoadExtensionList(vncExtensions, ARRAY_SIZE(vncExtensions), TRUE);
+    }
 
 #if XORG_AT_LEAST(1, 20, 0)
     xorgGlxCreateVendor();
@@ -1203,11 +1215,13 @@ DDXRingBell(int percent, int pitch, int duration)
         vncBell();
 }
 
+#if XORG_OLDER_THAN(1, 21, 1)
 Bool
 LegalModifier(unsigned int key, DeviceIntPtr pDev)
 {
     return TRUE;
 }
+#endif
 
 void
 ProcessInputEvents(void)
@@ -1234,3 +1248,9 @@ vncClientGone(int fd)
         GiveUp(0);
     }
 }
+
+int
+main(int argc, char *argv[], char *envp[])
+{
+    return dix_main(argc, argv, envp);
+}
