@@ -1,5 +1,5 @@
 /* Copyright (C) 2002-2005 RealVNC Ltd.  All Rights Reserved.
- * Copyright 2011 Pierre Ossman <ossman@cendio.se> for Cendio AB
+ * Copyright 2011-2025 Pierre Ossman <ossman@cendio.se> for Cendio AB
  * 
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,11 +26,16 @@
 #include <assert.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
+#include <unistd.h>
 #include <sys/time.h>
 
-#include <rfb/LogWriter.h>
+#include <core/LogWriter.h>
+#include <core/string.h>
+#include <core/time.h>
+
 #include <rfb/CMsgWriter.h>
-#include <rfb/util.h>
+#include <rfb/ScreenSet.h>
 
 #include "DesktopWindow.h"
 #include "OptionsDialog.h"
@@ -41,6 +46,7 @@
 #include "Surface.h"
 #include "Viewport.h"
 #include "touch.h"
+#include "fltk/event_dispatch_handler.h"
 
 #include <FL/Fl.H>
 #include <FL/Fl_Image_Surface.H>
@@ -58,8 +64,8 @@
 #endif
 
 // width of each "edge" region where scrolling happens,
-// as a ratio compared to the viewport size
-// default: 1/16th of the viewport size
+// as a ratio compared to the window size
+// default: 1/16th of the window size
 #define EDGE_SCROLL_SIZE 16
 // edge width is calculated at runtime; these values are just examples
 static int edge_scroll_size_x = 128;
@@ -70,32 +76,32 @@ static int edge_scroll_size_y = 96;
 // default: roughly 60 fps for smooth motion
 #define EDGE_SCROLL_SECONDS_PER_FRAME 0.016666
 
-using namespace rfb;
+// Time before we show an overlay tip again
+const time_t OVERLAY_REPEAT_TIMEOUT = 600;
 
-static rfb::LogWriter vlog("DesktopWindow");
+static core::LogWriter vlog("DesktopWindow");
 
 // Global due to http://www.fltk.org/str.php?L2177 and the similar
 // issue for Fl::event_dispatch.
 static std::set<DesktopWindow *> instances;
 
-DesktopWindow::DesktopWindow(int w, int h, const char *name,
-                             const rfb::PixelFormat& serverPF,
-                             CConn* cc_)
-  : Fl_Window(w, h), cc(cc_), offscreen(NULL), overlay(NULL),
+DesktopWindow::DesktopWindow(int w, int h, CConn* cc_)
+  : Fl_Window(w, h), cc(cc_), offscreen(nullptr),
     firstUpdate(true),
-    delayedFullscreen(false), delayedDesktopSize(false),
-    keyboardGrabbed(false), mouseGrabbed(false),
+    delayedFullscreen(false), sentDesktopSize(false),
+    pendingRemoteResize(false), lastResize({0, 0}),
+    keyboardGrabbed(false), mouseGrabbed(false), regrabOnFocus(false),
     statsLastUpdates(0), statsLastPixels(0), statsLastPosition(0),
-    statsGraph(NULL)
+    statsGraph(nullptr)
 {
   Fl_Group* group;
 
   // Dummy group to prevent FLTK from moving our widgets around
   group = new Fl_Group(0, 0, w, h);
-  group->resizable(NULL);
+  group->resizable(nullptr);
   resizable(group);
 
-  viewport = new Viewport(w, h, serverPF, cc);
+  viewport = new Viewport(w, h, cc);
 
   // Position will be adjusted later
   hscroll = new Fl_Scrollbar(0, 0, 0, 0);
@@ -108,7 +114,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 
   callback(handleClose, this);
 
-  setName(name);
+  updateCaption();
 
   OptionsDialog::addCallback(handleOptions, this);
 
@@ -118,7 +124,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   instances.insert(this);
 
   // Hack. See below...
-  Fl::event_dispatch(fltkDispatch);
+  fl_add_event_dispatch(fltkDispatch, this);
 
   // Support for -geometry option. Note that although we do support
   // negative coordinates, we do not support -XOFF-YOFF (ie
@@ -174,7 +180,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 #ifdef __APPLE__
   // On OS X we can do the maximize thing properly before the
   // window is showned. Other platforms handled further down...
-  if (maximize) {
+  if (::maximize) {
     int dummy;
     Fl::screen_work_area(dummy, dummy, w, h, geom_x, geom_y);
   }
@@ -199,6 +205,11 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 
   show();
 
+#ifdef __APPLE__
+  // FLTK does its own full screen, so disable the system one
+  cocoa_prevent_native_fullscreen(this);
+#endif
+
   // Full screen events are not sent out for a hidden window,
   // so send a fake one here to set up things properly.
   if (fullscreen_active())
@@ -208,7 +219,7 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   // maximized property on Windows and X11 before showing the window.
   // See STR #2083 and STR #2178
 #ifndef __APPLE__
-  if (maximize) {
+  if (::maximize) {
     maximizeWindow();
   }
 #endif
@@ -216,22 +227,22 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
   // Adjust layout now that we're visible and know our final size
   repositionWidgets();
 
-  if (delayedFullscreen) {
-    // Hack: Fullscreen requests may be ignored, so we need a timeout for
-    // when we should stop waiting. We also really need to wait for the
-    // resize, which can come after the fullscreen event.
-    Fl::add_timeout(0.5, handleFullscreenTimeout, this);
-    fullscreen_on();
-  }
-
   // Throughput graph for debugging
-  if (vlog.getLevel() >= LogWriter::LEVEL_DEBUG) {
+  if (vlog.getLevel() >= core::LogWriter::LEVEL_DEBUG) {
     memset(&stats, 0, sizeof(stats));
     Fl::add_timeout(0, handleStatsTimeout, this);
   }
 
-  // Show hint about menu key
-  Fl::add_timeout(0.5, menuOverlay, this);
+  // Show hint about menu shortcut
+  unsigned modifierMask;
+
+  modifierMask = 0;
+  for (core::EnumListEntry key : shortcutModifiers)
+    modifierMask |= ShortcutHandler::parseModifier(key.getValueStr().c_str());
+
+  if (modifierMask)
+    addOverlayTip(_("Press %sM to open the context menu"),
+                  ShortcutHandler::modifierPrefix(modifierMask));
 
   // By default we get a slight delay when we warp the pointer, something
   // we don't want or we'll get jerky movement
@@ -245,19 +256,26 @@ DesktopWindow::DesktopWindow(int w, int h, const char *name,
 
 DesktopWindow::~DesktopWindow()
 {
+  // Don't leave any dangling grabs as they are not automatically
+  // cleaned up on all platforms
+  ungrabPointer();
+  ungrabKeyboard();
+
   // Unregister all timeouts in case they get a change tro trigger
   // again later when this object is already gone.
-  Fl::remove_timeout(handleGrab, this);
   Fl::remove_timeout(handleResizeTimeout, this);
   Fl::remove_timeout(handleFullscreenTimeout, this);
   Fl::remove_timeout(handleEdgeScroll, this);
   Fl::remove_timeout(handleStatsTimeout, this);
-  Fl::remove_timeout(menuOverlay, this);
   Fl::remove_timeout(updateOverlay, this);
+  Fl::remove_idle(checkFocus, this);
 
   OptionsDialog::removeCallback(handleOptions);
 
-  delete overlay;
+  while (!overlays.empty()) {
+    delete overlays.front().surface;
+    overlays.pop_front();
+  }
   delete offscreen;
 
   delete statsGraph;
@@ -267,7 +285,7 @@ DesktopWindow::~DesktopWindow()
   if (instances.size() == 0)
     Fl::remove_handler(fltkHandle);
 
-  Fl::event_dispatch(Fl::handle_);
+  fl_remove_event_dispatch(fltkDispatch, this);
 
   // FLTK automatically deletes all child widgets, so we shouldn't touch
   // them ourselves here
@@ -280,13 +298,50 @@ const rfb::PixelFormat &DesktopWindow::getPreferredPF()
 }
 
 
-void DesktopWindow::setName(const char *name)
+void DesktopWindow::updateCaption()
 {
-  char windowNameStr[256];
+  const size_t maxLen = 100;
+  std::string windowName;
+  const char *labelFormat;
+  size_t maxNameSize;
+  std::string name;
 
-  snprintf(windowNameStr, 256, "%.240s - TigerVNC", name);
+  // FIXME: All of this consideres bytes, not characters
 
-  copy_label(windowNameStr);
+  if (keyboardGrabbed)
+    labelFormat = _("%s - TigerVNC (keyboard grabbed)");
+  else
+    labelFormat = _("%s - TigerVNC");
+
+  // Ignore the length of '%s' since it is
+  // a format marker which won't take up space
+  maxNameSize = maxLen - strlen(labelFormat) + 2;
+
+  name = cc->server.name();
+
+  if (name.size() > maxNameSize) {
+    if (maxNameSize <= strlen("...")) {
+      // Even an ellipsis won't fit
+      name.clear();
+    }
+    else {
+      int offset;
+
+      // We need to truncate, add an ellipsis
+      offset = maxNameSize - strlen("...");
+      name.resize(offset);
+      name += "...";
+    }
+  }
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+
+  windowName = core::format(labelFormat, name.c_str());
+
+#pragma GCC diagnostic pop
+
+  copy_label(windowName.c_str());
 }
 
 
@@ -296,15 +351,8 @@ void DesktopWindow::setName(const char *name)
 void DesktopWindow::updateWindow()
 {
   if (firstUpdate) {
-    if (cc->server.supportsSetDesktopSize) {
-      // Hack: Wait until we're in the proper mode and position until
-      // resizing things, otherwise we might send the wrong thing.
-      if (delayedFullscreen)
-        delayedDesktopSize = true;
-      else
-        handleDesktopSize();
-    }
     firstUpdate = false;
+    remoteResize();
   }
 
   viewport->updateWindow();
@@ -342,9 +390,9 @@ void DesktopWindow::resizeFramebuffer(int new_w, int new_h)
 
   XGetWindowProperty(fl_display, fl_xid(this), net_wm_state, 0, 1024, False, XA_ATOM, &type, &format, &nitems, &remain, (unsigned char**)&atoms);
 
-  for (unsigned long i = 0;i < nitems;i++) {
-    if ((atoms[i] == net_wm_state_maximized_vert) ||
-        (atoms[i] == net_wm_state_maximized_horz)) {
+  for (unsigned long n = 0;n < nitems;n++) {
+    if ((atoms[n] == net_wm_state_maximized_vert) ||
+        (atoms[n] == net_wm_state_maximized_horz)) {
       maximized = true;
       break;
     }
@@ -359,13 +407,6 @@ void DesktopWindow::resizeFramebuffer(int new_w, int new_h)
   if (!fullscreen_active() && !maximized) {
     if ((w() == viewport->w()) && (h() == viewport->h()))
       size(new_w, new_h);
-    else {
-      // Make sure the window isn't too big. We do this manually because
-      // we have to disable the window size restriction (and it isn't
-      // entirely trustworthy to begin with).
-      if ((w() > new_w) || (h() > new_h))
-        size(__rfbmin(w(), new_w), __rfbmin(h(), new_h));
-    }
   }
 
   viewport->size(new_w, new_h);
@@ -374,15 +415,26 @@ void DesktopWindow::resizeFramebuffer(int new_w, int new_h)
 }
 
 
-void DesktopWindow::setCursor(int width, int height,
-                              const rfb::Point& hotspot,
-                              const uint8_t* data)
+void DesktopWindow::setDesktopSizeDone(unsigned result)
 {
-  viewport->setCursor(width, height, hotspot, data);
+  pendingRemoteResize = false;
+
+  if (result != 0)
+    return;
+
+  // We might have resized again whilst waiting for the previous
+  // request, so check if we are in sync
+  remoteResize();
 }
 
 
-void DesktopWindow::setCursorPos(const rfb::Point& pos)
+void DesktopWindow::setCursor()
+{
+  viewport->setCursor();
+}
+
+
+void DesktopWindow::setCursorPos(const core::Point& pos)
 {
   if (!mouseGrabbed) {
     // Do nothing if we do not have the mouse captured.
@@ -438,7 +490,7 @@ void DesktopWindow::draw()
 #if !defined(__APPLE__)
 
   // Adjust offscreen surface dimensions
-  if ((offscreen == NULL) ||
+  if ((offscreen == nullptr) ||
       (offscreen->width() != w()) || (offscreen->height() != h())) {
     delete offscreen;
     offscreen = new Surface(w(), h());
@@ -500,21 +552,24 @@ void DesktopWindow::draw()
   }
 
   // Overlay (if active)
-  if (overlay) {
+  if (!overlays.empty()) {
     int ox, oy, ow, oh;
     int sx, sy, sw, sh;
+    struct Overlay overlay;
+
+    overlay = overlays.front();
 
     // Make sure it's properly seen by adjusting it relative to the
     // primary screen rather than the entire window
     if (fullscreen_active()) {
       assert(Fl::screen_count() >= 1);
 
-      rfb::Rect windowRect, screenRect;
+      core::Rect windowRect, screenRect;
       windowRect.setXYWH(x(), y(), w(), h());
 
       bool foundEnclosedScreen = false;
-      for (int i = 0; i < Fl::screen_count(); i++) {
-        Fl::screen_xywh(sx, sy, sw, sh, i);
+      for (int idx = 0; idx < Fl::screen_count(); idx++) {
+        Fl::screen_xywh(sx, sy, sw, sh, idx);
 
         // The screen with the smallest index that are enclosed by
         // the viewport will be used for showing the overlay.
@@ -540,18 +595,20 @@ void DesktopWindow::draw()
       sw = w();
     }
 
-    ox = X = sx + (sw - overlay->width()) / 2;
+    ox = X = sx + (sw - overlay.surface->width()) / 2;
     oy = Y = sy + 50;
-    ow = overlay->width();
-    oh = overlay->height();
+    ow = overlay.surface->width();
+    oh = overlay.surface->height();
 
     fl_clip_box(ox, oy, ow, oh, ox, oy, ow, oh);
 
     if ((ow != 0) && (oh != 0)) {
       if (offscreen)
-        overlay->blend(offscreen, ox - X, oy - Y, ox, oy, ow, oh, overlayAlpha);
+        overlay.surface->blend(offscreen, ox - X, oy - Y,
+                               ox, oy, ow, oh, overlay.alpha);
       else
-        overlay->blend(ox - X, oy - Y, ox, oy, ow, oh, overlayAlpha);
+        overlay.surface->blend(ox - X, oy - Y,
+                               ox, oy, ow, oh, overlay.alpha);
     }
   }
 
@@ -632,10 +689,10 @@ void DesktopWindow::resize(int x, int y, int w, int h)
     }
 
     if (resize_req) {
-      for (int i = 0;i < Fl::screen_count();i++) {
+      for (int idx = 0;idx < Fl::screen_count();idx++) {
         int sx, sy, sw, sh;
 
-        Fl::screen_xywh(sx, sy, sw, sh, i);
+        Fl::screen_xywh(sx, sy, sw, sh, idx);
 
         // We can't trust x and y if the window isn't mapped as the
         // window manager might adjust those numbers
@@ -661,51 +718,59 @@ void DesktopWindow::resize(int x, int y, int w, int h)
   Fl_Window::resize(x, y, w, h);
 
   if (resizing) {
-    // Try to get the remote size to match our window size, provided
-    // the following conditions are true:
-    //
-    // a) The user has this feature turned on
-    // b) The server supports it
-    // c) We're not still waiting for a chance to handle DesktopSize
-    // d) We're not still waiting for startup fullscreen to kick in
-    //
-    if (not firstUpdate and not delayedFullscreen and
-        ::remoteResize and cc->server.supportsSetDesktopSize) {
-      // We delay updating the remote desktop as we tend to get a flood
-      // of resize events as the user is dragging the window.
-      Fl::remove_timeout(handleResizeTimeout, this);
-      Fl::add_timeout(0.5, handleResizeTimeout, this);
-    }
+    remoteResize();
 
     repositionWidgets();
   }
-
-  // Some systems require a grab after the window size has been changed.
-  // Otherwise they might hold on to displays, resulting in them being unusable.
-  maybeGrabKeyboard();
 }
 
-
-void DesktopWindow::menuOverlay(void* data)
+void DesktopWindow::addOverlayTip(const char* text, ...)
 {
-  DesktopWindow *self;
+  va_list ap;
+  char textbuf[1024];
 
-  self = (DesktopWindow*)data;
+  std::map<std::string, time_t>::iterator iter;
 
-  if (strcmp((const char*)menuKey, "") != 0) {
-    self->setOverlay(_("Press %s to open the context menu"),
-                     (const char*)menuKey);
+  va_start(ap, text);
+  vsnprintf(textbuf, sizeof(textbuf), text, ap);
+  textbuf[sizeof(textbuf)-1] = '\0';
+  va_end(ap);
+
+  // Purge all old entries
+  for (iter = overlayTimes.begin(); iter != overlayTimes.end(); ) {
+    if ((time(nullptr) - iter->second) >= OVERLAY_REPEAT_TIMEOUT)
+      overlayTimes.erase(iter++);
+    else
+      iter++;
   }
+
+  // Recently shown?
+  if (overlayTimes.count(textbuf) > 0)
+    return;
+
+  overlayTimes[textbuf] = time(nullptr);
+
+  addOverlay(textbuf);
 }
 
-void DesktopWindow::setOverlay(const char* text, ...)
+void DesktopWindow::addOverlayError(const char* text, ...)
+{
+  va_list ap;
+  char textbuf[1024];
+
+  va_start(ap, text);
+  vsnprintf(textbuf, sizeof(textbuf), text, ap);
+  textbuf[sizeof(textbuf)-1] = '\0';
+  va_end(ap);
+
+  addOverlay(textbuf);
+}
+
+void DesktopWindow::addOverlay(const char *text)
 {
   const Fl_Fontsize fontsize = 16;
   const int margin = 10;
 
-  va_list ap;
-  char textbuf[1024];
-
   Fl_Image_Surface *surface;
 
   Fl_RGB_Image* imageText;
@@ -719,13 +784,7 @@ void DesktopWindow::setOverlay(const char* text, ...)
   unsigned char* a;
   const unsigned char* b;
 
-  delete overlay;
-  Fl::remove_timeout(updateOverlay, this);
-
-  va_start(ap, text);
-  vsnprintf(textbuf, sizeof(textbuf), text, ap);
-  textbuf[sizeof(textbuf)-1] = '\0';
-  va_end(ap);
+  struct Overlay overlay;
 
 #if !defined(WIN32) && !defined(__APPLE__)
   // FLTK < 1.3.5 crashes if fl_gc is unset
@@ -735,7 +794,7 @@ void DesktopWindow::setOverlay(const char* text, ...)
 
   fl_font(FL_HELVETICA, fontsize);
   w = 0;
-  fl_measure(textbuf, w, h);
+  fl_measure(text, w, h);
 
   // Margins
   w += margin * 2 * 2;
@@ -748,7 +807,7 @@ void DesktopWindow::setOverlay(const char* text, ...)
 
   fl_font(FL_HELVETICA, fontsize);
   fl_color(FL_WHITE);
-  fl_draw(textbuf, 0, 0, w, h, FL_ALIGN_CENTER);
+  fl_draw(text, 0, 0, w, h, FL_ALIGN_CENTER);
 
   imageText = surface->image();
   delete surface;
@@ -784,39 +843,53 @@ void DesktopWindow::setOverlay(const char* text, ...)
 
   delete imageText;
 
-  overlay = new Surface(image);
-  overlayAlpha = 0;
-  gettimeofday(&overlayStart, NULL);
+  overlay.surface = new Surface(image);
+  overlay.alpha = 0;
+  memset(&overlay.start, 0, sizeof(overlay.start));
+  overlays.push_back(overlay);
 
   delete image;
   delete [] buffer;
 
-  Fl::add_timeout(1.0/60, updateOverlay, this);
+  if (overlays.size() == 1)
+    Fl::add_timeout(0.5, updateOverlay, this);
 }
 
 void DesktopWindow::updateOverlay(void *data)
 {
   DesktopWindow *self;
+  struct Overlay* overlay;
   unsigned elapsed;
 
   self = (DesktopWindow*)data;
 
-  elapsed = msSince(&self->overlayStart);
+  if (self->overlays.empty())
+    return;
+
+  overlay = &self->overlays.front();
+
+  if (overlay->start.tv_sec == 0)
+    gettimeofday(&overlay->start, nullptr);
+
+  elapsed = core::msSince(&overlay->start);
 
   if (elapsed < 500) {
-    self->overlayAlpha = (unsigned)255 * elapsed / 500;
+    overlay->alpha = (unsigned)255 * elapsed / 500;
     Fl::add_timeout(1.0/60, updateOverlay, self);
   } else if (elapsed < 3500) {
-    self->overlayAlpha = 255;
+    overlay->alpha = 255;
     Fl::add_timeout(3.0, updateOverlay, self);
   } else if (elapsed < 4000) {
-    self->overlayAlpha = (unsigned)255 * (4000 - elapsed) / 500;
+    overlay->alpha = (unsigned)255 * (4000 - elapsed) / 500;
     Fl::add_timeout(1.0/60, updateOverlay, self);
   } else {
-    delete self->overlay;
-    self->overlay = NULL;
+    delete overlay->surface;
+    self->overlays.pop_front();
+    if (!self->overlays.empty())
+      Fl::add_timeout(0.5, updateOverlay, self);
   }
 
+  // FIXME: Only damage relevant area
   self->damage(FL_DAMAGE_USER1);
 }
 
@@ -830,10 +903,44 @@ int DesktopWindow::handle(int event)
     // Update scroll bars
     repositionWidgets();
 
-    if (fullscreen_active())
-      maybeGrabKeyboard();
-    else
-      ungrabKeyboard();
+    // Show how to get out of full screen
+    if (fullscreen_active()) {
+      unsigned modifierMask;
+
+      modifierMask = 0;
+      for (core::EnumListEntry key : shortcutModifiers)
+        modifierMask |= ShortcutHandler::parseModifier(key.getValueStr().c_str());
+
+      if (modifierMask)
+        addOverlayTip(_("Press %sEnter to leave full-screen mode"),
+                      ShortcutHandler::modifierPrefix(modifierMask));
+    }
+
+#ifdef __APPLE__
+    // Complain to the user if we won't have permission to grab keyboard
+    if (fullscreenSystemKeys && fullscreen_active()) {
+      // FIXME: There is some race during initial full screen where we
+      //        fail to give focus to the popup, but we can work around
+      //        it using a timer
+      Fl::add_timeout(0, [](void*) { cocoa_is_trusted(true); }, nullptr);
+    }
+#endif
+
+    // Automatically toggle keyboard grab?
+    if (fullscreenSystemKeys) {
+      if (fullscreen_active())
+        grabKeyboard();
+      else
+        ungrabKeyboard();
+    }
+
+    // The window manager respected our full screen request, so stop
+    // waiting and delaying the session resize
+    if (delayedFullscreen && fullscreen_active()) {
+      Fl::remove_timeout(handleFullscreenTimeout, this);
+      delayedFullscreen = false;
+      remoteResize();
+    }
 
     break;
 
@@ -865,9 +972,9 @@ int DesktopWindow::handle(int event)
     }
     if (fullscreen_active()) {
       // calculate width of "edge" regions
-      edge_scroll_size_x = viewport->w() / EDGE_SCROLL_SIZE;
-      edge_scroll_size_y = viewport->h() / EDGE_SCROLL_SIZE;
-      // if cursor is near the edge of the viewport, scroll
+      edge_scroll_size_x = w() / EDGE_SCROLL_SIZE;
+      edge_scroll_size_y = h() / EDGE_SCROLL_SIZE;
+      // if cursor is near the edge of the window, scroll
       if (((viewport->x() < 0) && (Fl::event_x() < edge_scroll_size_x)) ||
           ((viewport->x() + viewport->w() >= w()) && (Fl::event_x() >= w() - edge_scroll_size_x)) ||
           ((viewport->y() < 0) && (Fl::event_y() < edge_scroll_size_y)) ||
@@ -884,37 +991,48 @@ int DesktopWindow::handle(int event)
 }
 
 
-int DesktopWindow::fltkDispatch(int event, Fl_Window *win)
+int DesktopWindow::fltkDispatch(int event, Fl_Window *win, void *)
 {
-  int ret;
-
   // FLTK keeps spamming bogus FL_MOVE events if _any_ X event is
   // received with the mouse pointer outside our windows
   // https://github.com/fltk/fltk/issues/76
-  if ((event == FL_MOVE) && (win == NULL))
+  if ((event == FL_MOVE) && (win == nullptr))
     return 0;
 
-  ret = Fl::handle_(event, win);
-
-  // This is hackish and the result of the dodgy focus handling in FLTK.
-  // The basic problem is that FLTK's view of focus and the system's tend
-  // to differ, and as a result we do not see all the FL_FOCUS events we
-  // need. Fortunately we can grab them here...
+#if !defined(WIN32) && !defined(__APPLE__)
+  // FLTK passes through the fake grab focus events that can cause us
+  // to end up in an infinite loop
+  // https://github.com/fltk/fltk/issues/295
+  if ((event == FL_FOCUS) || (event == FL_UNFOCUS)) {
+    const XFocusChangeEvent* xfocus = &fl_xevent->xfocus;
+    if ((xfocus->mode == NotifyGrab) || (xfocus->mode == NotifyUngrab))
+      return 0;
+  }
+#endif
 
   DesktopWindow *dw = dynamic_cast<DesktopWindow*>(win);
 
   if (dw) {
     switch (event) {
-    // Focus might not stay with us just because we have grabbed the
-    // keyboard. E.g. we might have sub windows, or we're not using
-    // all monitors and the user clicked on another application.
-    // Make sure we update our grabs with the focus changes.
+    // This is hackish and the result of the dodgy focus handling in FLTK.
+    // The basic problem is that FLTK's view of focus and the system's tend
+    // to differ, and as a result we do not see all the FL_FOCUS events we
+    // need. Fortunately we can grab them here...
     case FL_FOCUS:
-      dw->maybeGrabKeyboard();
-      break;
     case FL_UNFOCUS:
-      if (fullscreenSystemKeys) {
-        dw->ungrabKeyboard();
+      Fl::add_idle(checkFocus, dw);
+      break;
+
+    case FL_SHOW:
+      // In this particular place, FL_SHOW means an actual MapNotify,
+      // which means we can continue enabling initial fullscreen.
+      if (dw->delayedFullscreen) {
+        // Hack: Fullscreen requests may be ignored, so we need a
+        // timeout for when we should stop waiting. We also really need
+        // to wait for the resize, which can come after the fullscreen
+        // event.
+        Fl::add_timeout(0.5, handleFullscreenTimeout, dw);
+        dw->fullscreen_on();
       }
       break;
 
@@ -930,7 +1048,7 @@ int DesktopWindow::fltkDispatch(int event, Fl_Window *win)
     }
   }
 
-  return ret;
+  return 0;
 }
 
 int DesktopWindow::fltkHandle(int event)
@@ -943,14 +1061,6 @@ int DesktopWindow::fltkHandle(int event)
     // not be resized to cover the new screen. A timer makes sense
     // also on other systems, to make sure that whatever desktop
     // environment has a chance to deal with things before we do.
-    // Please note that when using FullscreenSystemKeys on macOS, the
-    // display configuration cannot be changed: macOS will not detect
-    // added or removed screens and there will be no
-    // FL_SCREEN_CONFIGURATION_CHANGED event. This is by design:
-    // "When you capture a display, you have exclusive use of the
-    // display. Other applications and system services are not allowed
-    // to use the display or change its configuration. In addition,
-    // they are not notified of display changes"
     Fl::remove_timeout(reconfigureFullscreen);
     Fl::add_timeout(0.5, reconfigureFullscreen);
   }
@@ -960,8 +1070,8 @@ int DesktopWindow::fltkHandle(int event)
 
 void DesktopWindow::fullscreen_on()
 {
-  bool allMonitors = !strcasecmp(fullScreenMode, "all");
-  bool selectedMonitors = !strcasecmp(fullScreenMode, "selected");
+  bool allMonitors = fullScreenMode == "all";
+  bool selectedMonitors = fullScreenMode == "selected";
   int top, bottom, left, right;
 
   if (not selectedMonitors and not allMonitors) {
@@ -974,12 +1084,11 @@ void DesktopWindow::fullscreen_on()
     std::set<int> monitors;
 
     if (selectedMonitors and not allMonitors) {
-      std::set<int> selected = fullScreenSelectedMonitors.getParam();
+      std::set<int> selected = fullScreenSelectedMonitors.getMonitors();
       monitors.insert(selected.begin(), selected.end());
     } else {
-      for (int i = 0; i < Fl::screen_count(); i++) {
-        monitors.insert(i);
-      }
+      for (int idx = 0; idx < Fl::screen_count(); idx++)
+        monitors.insert(idx);
     }
 
     // If no monitors were found in the selected monitors case, we want
@@ -1027,40 +1136,13 @@ void DesktopWindow::fullscreen_on()
     }
 
   }
-#ifdef __APPLE__
-  // This is a workaround for a bug in FLTK, see: https://github.com/fltk/fltk/pull/277
-  int savedLevel;
-  savedLevel = cocoa_get_level(this);
-#endif
+
   fullscreen_screens(top, bottom, left, right);
-#ifdef __APPLE__
-  // This is a workaround for a bug in FLTK, see: https://github.com/fltk/fltk/pull/277
-  if (cocoa_get_level(this) != savedLevel)
-    cocoa_set_level(this, savedLevel);
-#endif
 
   if (!fullscreen_active())
     fullscreen();
 }
 
-#if !defined(WIN32) && !defined(__APPLE__)
-Bool eventIsFocusWithSerial(Display* /*display*/, XEvent *event,
-                            XPointer arg)
-{
-  unsigned long serial;
-
-  serial = *(unsigned long*)arg;
-
-  if (event->xany.serial != serial)
-    return False;
-
-  if ((event->type != FocusIn) && (event->type != FocusOut))
-    return False;
-
-  return True;
-}
-#endif
-
 bool DesktopWindow::hasFocus()
 {
   Fl_Widget* focus;
@@ -1075,66 +1157,89 @@ bool DesktopWindow::hasFocus()
   return focus->window() == this;
 }
 
-void DesktopWindow::maybeGrabKeyboard()
+void DesktopWindow::checkFocus(void *data)
 {
-  if (fullscreenSystemKeys && fullscreen_active() && hasFocus())
-    grabKeyboard();
+  Fl::remove_idle(checkFocus, data);
+
+  DesktopWindow *dw = (DesktopWindow*)data;
+  // Focus might not stay with us just because we have grabbed the
+  // keyboard. E.g. we might have sub windows, or the user clicked on
+  // another application. Make sure we update our grabs with the focus
+  // changes.
+  if (!dw->hasFocus()) {
+    // If the grab is active when we lose focus, the user likely wants
+    // the grab to remain once we regain focus
+    if (dw->keyboardGrabbed)
+      dw->regrabOnFocus = true;
+    dw->ungrabKeyboard();
+  } else {
+    if (dw->regrabOnFocus ||
+        (fullscreenSystemKeys && dw->fullscreen_active()))
+      dw->grabKeyboard();
+    dw->regrabOnFocus = false;
+  }
 }
 
 void DesktopWindow::grabKeyboard()
 {
+  unsigned modifierMask;
+
   // Grabbing the keyboard is fairly safe as FLTK reroutes events to the
   // correct widget regardless of which low level window got the system
   // event.
 
   // FIXME: Push this stuff into FLTK.
 
+  if (keyboardGrabbed)
+    return;
+
+  if (!hasFocus())
+    return;
+
 #if defined(WIN32)
   int ret;
   
   ret = win32_enable_lowlevel_keyboard(fl_xid(this));
   if (ret != 0) {
-    vlog.error(_("Failure grabbing keyboard"));
+    vlog.error(_("Failure grabbing control of the keyboard"));
+    addOverlayError(_("Failure grabbing control of the keyboard"));
     return;
   }
 #elif defined(__APPLE__)
-  int ret;
-  
-  ret = cocoa_capture_displays(this);
-  if (ret != 0) {
-    vlog.error(_("Failure grabbing keyboard"));
+  bool ret;
+
+  ret = cocoa_tap_keyboard();
+  if (!ret) {
+    vlog.error(_("Failure grabbing control of the keyboard"));
+    addOverlayError(_("Failure grabbing control of the keyboard"));
     return;
   }
 #else
   int ret;
 
-  XEvent xev;
-  unsigned long serial;
-
-  serial = XNextRequest(fl_display);
-
   ret = XGrabKeyboard(fl_display, fl_xid(this), True,
                       GrabModeAsync, GrabModeAsync, CurrentTime);
   if (ret) {
     if (ret == AlreadyGrabbed) {
-      // It seems like we can race with the WM in some cases.
-      // Try again in a bit.
-      if (!Fl::has_timeout(handleGrab, this))
-        Fl::add_timeout(0.500, handleGrab, this);
-    } else {
-      vlog.error(_("Failure grabbing keyboard"));
+      // It seems like we can race with the WM in some cases, e.g. when
+      // the WM holds the keyboard as part of handling Alt+Tab.
+      // Repeat the request a few times and see if we get it...
+      for (int attempt = 0; attempt < 5; attempt++) {
+        usleep(100000);
+        // Also throttle based on how busy the X server is
+        XSync(fl_display, False);
+        ret = XGrabKeyboard(fl_display, fl_xid(this), True,
+                            GrabModeAsync, GrabModeAsync, CurrentTime);
+        if (ret != AlreadyGrabbed)
+          break;
+      }
     }
-    return;
-  }
 
-  // Xorg 1.20+ generates FocusIn/FocusOut even when there is no actual
-  // change of focus. This causes us to get stuck in an endless loop
-  // grabbing and ungrabbing the keyboard. Avoid this by filtering out
-  // any focus events generated by XGrabKeyboard().
-  XSync(fl_display, False);
-  while (XCheckIfEvent(fl_display, &xev, &eventIsFocusWithSerial,
-                       (XPointer)&serial) == True) {
-    vlog.debug("Ignored synthetic focus event cause by grab change");
+    if (ret) {
+      vlog.error(_("Failure grabbing control of the keyboard"));
+      addOverlayError(_("Failure grabbing control of the keyboard"));
+      return;
+    }
   }
 #endif
 
@@ -1142,39 +1247,37 @@ void DesktopWindow::grabKeyboard()
 
   if (contains(Fl::belowmouse()))
     grabPointer();
+
+  updateCaption();
+
+  modifierMask = 0;
+  for (core::EnumListEntry key : shortcutModifiers)
+    modifierMask |= ShortcutHandler::parseModifier(key.getValueStr().c_str());
+
+  if (modifierMask)
+    addOverlayTip(_("Press %s to release keyboard control from the session"),
+                  ShortcutHandler::modifierPrefix(modifierMask, true));
 }
 
 
 void DesktopWindow::ungrabKeyboard()
 {
-  Fl::remove_timeout(handleGrab, this);
-
   keyboardGrabbed = false;
 
   ungrabPointer();
 
+  updateCaption();
+
 #if defined(WIN32)
   win32_disable_lowlevel_keyboard(fl_xid(this));
 #elif defined(__APPLE__)
-  cocoa_release_displays(this);
+  cocoa_untap_keyboard();
 #else
   // FLTK has a grab so lets not mess with it
   if (Fl::grab())
     return;
 
-  XEvent xev;
-  unsigned long serial;
-
-  serial = XNextRequest(fl_display);
-
   XUngrabKeyboard(fl_display, CurrentTime);
-
-  // See grabKeyboard()
-  XSync(fl_display, False);
-  while (XCheckIfEvent(fl_display, &xev, &eventIsFocusWithSerial,
-                       (XPointer)&serial) == True) {
-    vlog.debug("Ignored synthetic focus event cause by grab change");
-  }
 #endif
 }
 
@@ -1205,16 +1308,6 @@ void DesktopWindow::ungrabPointer()
 }
 
 
-void DesktopWindow::handleGrab(void *data)
-{
-  DesktopWindow *self = (DesktopWindow*)data;
-
-  assert(self);
-
-  self->maybeGrabKeyboard();
-}
-
-
 #define _NET_WM_STATE_ADD           1  /* add/set property */
 void DesktopWindow::maximizeWindow()
 {
@@ -1255,32 +1348,13 @@ void DesktopWindow::maximizeWindow()
 }
 
 
-void DesktopWindow::handleDesktopSize()
-{
-  if (strcmp(desktopSize, "") != 0) {
-    int width, height;
-
-    // An explicit size has been requested
-
-    if (sscanf(desktopSize, "%dx%d", &width, &height) != 2)
-      return;
-
-    remoteResize(width, height);
-  } else if (::remoteResize) {
-    // No explicit size, but remote resizing is on so make sure it
-    // matches whatever size the window ended up being
-    remoteResize(w(), h());
-  }
-}
-
-
 void DesktopWindow::handleResizeTimeout(void *data)
 {
   DesktopWindow *self = (DesktopWindow *)data;
 
   assert(self);
 
-  self->remoteResize(self->w(), self->h());
+  self->remoteResize();
 }
 
 
@@ -1295,10 +1369,47 @@ void DesktopWindow::reconfigureFullscreen(void* /*data*/)
 }
 
 
-void DesktopWindow::remoteResize(int width, int height)
+void DesktopWindow::remoteResize()
 {
-  ScreenSet layout;
-  ScreenSet::const_iterator iter;
+  int width, height;
+  rfb::ScreenSet layout;
+  rfb::ScreenSet::const_iterator iter;
+
+  if (viewOnly)
+    return;
+
+  if (!::remoteResize)
+    return;
+  if (!cc->server.supportsSetDesktopSize)
+    return;
+
+  // Don't pester the server with a resize until we have our final size
+  if (delayedFullscreen)
+    return;
+
+  // Rate limit to one pending resize at a time
+  if (pendingRemoteResize)
+    return;
+
+  // And no more than once every 100ms
+  if (core::msSince(&lastResize) < 100) {
+    Fl::remove_timeout(handleResizeTimeout, this);
+    Fl::add_timeout((100.0 - core::msSince(&lastResize)) / 1000.0,
+                    handleResizeTimeout, this);
+    return;
+  }
+
+  width = w();
+  height = h();
+
+  if (!sentDesktopSize && (strcmp(desktopSize, "") != 0)) {
+    // An explicit size has been requested
+
+    if (sscanf(desktopSize, "%dx%d", &width, &height) != 2)
+      return;
+
+    sentDesktopSize = true;
+  }
 
   if (!fullscreen_active() || (width > w()) || (height > h())) {
     // In windowed mode (or the framebuffer is so large that we need
@@ -1332,10 +1443,9 @@ void DesktopWindow::remoteResize(int width, int height)
     layout.begin()->dimensions.br.x = width;
     layout.begin()->dimensions.br.y = height;
   } else {
-    int i;
     uint32_t id;
     int sx, sy, sw, sh;
-    rfb::Rect viewport_rect, screen_rect;
+    core::Rect viewport_rect, screen_rect;
 
     // In full screen we report all screens that are fully covered.
 
@@ -1348,8 +1458,8 @@ void DesktopWindow::remoteResize(int width, int height)
     // FIXME: We should really track screens better so we can handle
     //        a resized one.
     //
-    for (i = 0;i < Fl::screen_count();i++) {
-      Fl::screen_xywh(sx, sy, sw, sh, i);
+    for (int idx = 0;idx < Fl::screen_count();idx++) {
+      Fl::screen_xywh(sx, sy, sw, sh, idx);
 
       // Check that the screen is fully inside the framebuffer
       screen_rect.setXYWH(sx, sy, sw, sh);
@@ -1419,6 +1529,8 @@ void DesktopWindow::remoteResize(int width, int height)
     vlog.debug("%s", buffer);
   }
 
+  pendingRemoteResize = true;
+  gettimeofday(&lastResize, nullptr);
   cc->writer()->writeSetDesktopSize(width, height, layout);
 }
 
@@ -1519,11 +1631,6 @@ void DesktopWindow::handleOptions(void *data)
 {
   DesktopWindow *self = (DesktopWindow*)data;
 
-  if (fullscreenSystemKeys)
-    self->maybeGrabKeyboard();
-  else
-    self->ungrabKeyboard();
-
   // Call fullscreen_on even if active since it handles
   // fullScreenMode
   if (fullScreen)
@@ -1539,11 +1646,7 @@ void DesktopWindow::handleFullscreenTimeout(void *data)
   assert(self);
 
   self->delayedFullscreen = false;
-
-  if (self->delayedDesktopSize) {
-    self->handleDesktopSize();
-    self->delayedDesktopSize = false;
-  }
+  self->remoteResize();
 }
 
 void DesktopWindow::scrollTo(int x, int y)
@@ -1648,7 +1751,7 @@ void DesktopWindow::handleStatsTimeout(void *data)
   updates = self->cc->getUpdateCount();
   pixels = self->cc->getPixelCount();
   pos = self->cc->getPosition();
-  elapsed = msSince(&self->statsLastTime);
+  elapsed = core::msSince(&self->statsLastTime);
   if (elapsed < 1)
     elapsed = 1;
 
@@ -1658,7 +1761,7 @@ void DesktopWindow::handleStatsTimeout(void *data)
   self->stats[statsCount-1].pps = (pixels - self->statsLastPixels) * 1000 / elapsed;
   self->stats[statsCount-1].bps = (pos - self->statsLastPosition) * 1000 / elapsed;
 
-  gettimeofday(&self->statsLastTime, NULL);
+  gettimeofday(&self->statsLastTime, nullptr);
   self->statsLastUpdates = updates;
   self->statsLastPixels = pixels;
   self->statsLastPosition = pos;
@@ -1723,11 +1826,11 @@ void DesktopWindow::handleStatsTimeout(void *data)
   fl_draw(buffer, 5, statsHeight - 5);
 
   fl_color(FL_YELLOW);
-  fl_draw(siPrefix(self->stats[statsCount-1].pps, "pix/s").c_str(),
+  fl_draw(core::siPrefix(self->stats[statsCount-1].pps, "pix/s").c_str(),
           5 + (statsWidth-10)/3, statsHeight - 5);
 
   fl_color(FL_RED);
-  fl_draw(siPrefix(self->stats[statsCount-1].bps * 8, "bps").c_str(),
+  fl_draw(core::siPrefix(self->stats[statsCount-1].bps * 8, "bps").c_str(),
           5 + (statsWidth-10)*2/3, statsHeight - 5);
 
   image = surface->image();
