@@ -29,27 +29,25 @@ namespace rfb {
     class DesktopPage : public PropSheetPage {
     public:
       DesktopPage(const RegKey& rk)
-        : PropSheetPage(GetModuleHandle(0), MAKEINTRESOURCE(IDD_DESKTOP)), regKey(rk) {}
-      void initDialog() {
-        const char *action(rfb::win32::SDisplay::disconnectAction);
-        bool disconnectLock = stricmp(action, "Lock") == 0;
-        bool disconnectLogoff = stricmp(action, "Logoff") == 0;
+        : PropSheetPage(GetModuleHandle(nullptr), MAKEINTRESOURCE(IDD_DESKTOP)), regKey(rk) {}
+      void initDialog() override {
+        bool disconnectLock = rfb::win32::SDisplay::disconnectAction == "Lock";
+        bool disconnectLogoff = rfb::win32::SDisplay::disconnectAction == "Logoff";
         setItemChecked(IDC_DISCONNECT_LOGOFF, disconnectLogoff);
         setItemChecked(IDC_DISCONNECT_LOCK, disconnectLock);
         setItemChecked(IDC_DISCONNECT_NONE, !disconnectLock && !disconnectLogoff);
         setItemChecked(IDC_REMOVE_WALLPAPER, rfb::win32::SDisplay::removeWallpaper);
         setItemChecked(IDC_DISABLE_EFFECTS, rfb::win32::SDisplay::disableEffects);
       }
-      bool onCommand(int id, int /*cmd*/) {
+      bool onCommand(int id, int /*cmd*/) override {
         switch (id) {
         case IDC_DISCONNECT_LOGOFF:
         case IDC_DISCONNECT_LOCK:
         case IDC_DISCONNECT_NONE:
         case IDC_REMOVE_WALLPAPER:
         case IDC_DISABLE_EFFECTS:
-          const char *action(rfb::win32::SDisplay::disconnectAction);
-          bool disconnectLock = stricmp(action, "Lock") == 0;
-          bool disconnectLogoff = stricmp(action, "Logoff") == 0;
+          bool disconnectLock = rfb::win32::SDisplay::disconnectAction == "Lock";
+          bool disconnectLogoff = rfb::win32::SDisplay::disconnectAction == "Logoff";
           setChanged((disconnectLogoff != isItemChecked(IDC_DISCONNECT_LOGOFF)) ||
                      (disconnectLock != isItemChecked(IDC_DISCONNECT_LOCK)) ||
                      (isItemChecked(IDC_REMOVE_WALLPAPER) != rfb::win32::SDisplay::removeWallpaper) ||
@@ -58,7 +56,7 @@ namespace rfb {
         }
         return false;
       }
-      bool onOk() {
+      bool onOk() override {
         const char* action = "None";
         if (isItemChecked(IDC_DISCONNECT_LOGOFF))
           action = "Logoff";
