@@ -22,12 +22,15 @@
 #include <string.h>
 
 #include "resource.h"
-#include <rfb/Logger_stdio.h>
-#include <rfb/LogWriter.h>
+
+#include <core/Logger_stdio.h>
+#include <core/LogWriter.h>
+
 #include <rfb_win32/Dialog.h>
 #include <rfb_win32/RegConfig.h>
 #include <rfb_win32/CurrentUser.h>
 
+using namespace core;
 using namespace rfb;
 using namespace rfb::win32;
 
@@ -61,15 +64,12 @@ processParams(int argc, char* argv[]) {
     } else if (strcasecmp(argv[i], "-user") == 0) {
       configKey = HKEY_CURRENT_USER;
     } else {
-      // Try to process <option>=<value>, or -<bool>
-      if (Configuration::setParam(argv[i], true))
+      int ret;
+
+      ret = Configuration::handleParamArg(argc, argv, i);
+      if (ret > 0) {
+        i += ret - 1;
         continue;
-      // Try to process -<option> <value>
-      if ((argv[i][0] == '-') && (i+1 < argc)) {
-        if (Configuration::setParam(&argv[i][1], argv[i+1], true)) {
-          i++;
-          continue;
-        }
       }
     }
   }
@@ -84,15 +84,12 @@ int WINAPI WinMain(HINSTANCE inst, HINSTANCE /*prev*/, char* /*cmdLine*/, int /*
   freopen("CONIN$","rb",stdin);
   freopen("CONOUT$","wb",stdout);
   freopen("CONOUT$","wb",stderr);
-  setbuf(stderr, 0);
+  setbuf(stderr, nullptr);
   initStdIOLoggers();
-  LogWriter vlog("main");
   logParams.setParam("*:stderr:100");
   vlog.info("Starting vncconfig applet");
 #endif
 
-  Configuration::enableServerParams();
-
   try {
     try {
       // Process command-line args
@@ -126,11 +123,11 @@ int WINAPI WinMain(HINSTANCE inst, HINSTANCE /*prev*/, char* /*cmdLine*/, int /*
 
         // Set the DACL, and don't allow the key to inherit its parent's DACL
         rootKey.setDACL(acl, false);
-      } catch (rdr::SystemException& e) {
+      } catch (core::win32_error& e) {
         // Something weird happens on NT 4.0 SP5 but I can't reproduce it on other
         // NT 4.0 service pack revisions.
         if (e.err == ERROR_INVALID_PARAMETER) {
-          MsgBox(0, "Windows reported an error trying to secure the VNC Server settings for this user.  "
+          MsgBox(nullptr, "Windows reported an error trying to secure the VNC server settings for this user.  "
                     "Your settings may not be secure!", MB_ICONWARNING | MB_OK);
         } else if (e.err != ERROR_CALL_NOT_IMPLEMENTED &&
                    e.err != ERROR_NOT_LOGGED_ON) {
@@ -159,29 +156,29 @@ int WINAPI WinMain(HINSTANCE inst, HINSTANCE /*prev*/, char* /*cmdLine*/, int /*
       HICON icon = (HICON)LoadImage(inst, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 0, 0, LR_SHARED);
 
       // Create the PropertySheet handler
-      const char* propSheetTitle = "VNC Server Properties (Service-Mode)";
+      const char* propSheetTitle = "VNC server properties (service-mode)";
       if (configKey == HKEY_CURRENT_USER)
-        propSheetTitle = "VNC Server Properties (User-Mode)";
+        propSheetTitle = "VNC server properties (user-mode)";
       PropSheet sheet(inst, propSheetTitle, pages, icon);
 
 #ifdef _DEBUG
-      vlog.debug("capture dialogs=%s", captureDialogs ? "true" : "false");
-      sheet.showPropSheet(0, true, false, captureDialogs);
+      vlog.debug("Capture dialogs=%s", captureDialogs ? "true" : "false");
+      sheet.showPropSheet(nullptr, true, false, captureDialogs);
 #else
-      sheet.showPropSheet(0, true, false);
+      sheet.showPropSheet(nullptr, true, false);
 #endif
-    } catch (rdr::SystemException& e) {
+    } catch (core::win32_error& e) {
       switch (e.err) {
       case ERROR_ACCESS_DENIED:
-        MsgBox(0, "You do not have sufficient access rights to run the VNC Configuration applet",
+        MsgBox(nullptr, "You do not have sufficient access rights to run the VNC Configuration applet",
                MB_ICONSTOP | MB_OK);
         return 1;
       };
       throw;
     }
 
-  } catch (rdr::Exception& e) {
-    MsgBox(NULL, e.str(), MB_ICONEXCLAMATION | MB_OK);
+  } catch (std::exception& e) {
+    MsgBox(nullptr, e.what(), MB_ICONEXCLAMATION | MB_OK);
     return 1;
   }
 
