@@ -27,15 +27,17 @@
 #include <winvnc/ListConnInfo.h>
 #include <winvnc/STrayIcon.h>
 
-#include <os/Mutex.h>
+#include <core/LogWriter.h>
+
+#include <network/TcpSocket.h>
 
 #include <rfb_win32/ComputerName.h>
 #include <rfb_win32/CurrentUser.h>
 #include <rfb_win32/Service.h>
 
-#include <rfb/Hostname.h>
-#include <rfb/LogWriter.h>
+#include <rfb/SConnection.h>
 
+using namespace core;
 using namespace rfb;
 using namespace win32;
 using namespace winvnc;
@@ -48,7 +50,8 @@ const char* winvnc::VNCServerWin32::RegConfigPath = "Software\\TigerVNC\\WinVNC4
 
 
 static IntParameter port_number("PortNumber",
-  "TCP/IP port on which the server will accept connections", 5900);
+  "TCP/IP port on which the server will accept connections",
+  5900, 0, 65535);
 static StringParameter hosts("Hosts",
   "Filter describing which hosts are allowed access to this server", "+");
 static BoolParameter localHost("LocalHost",
@@ -69,11 +72,6 @@ VNCServerWin32::VNCServerWin32()
     config(&sockMgr), rfbSock(&sockMgr), trayIcon(nullptr),
     queryConnectDialog(nullptr)
 {
-  commandLock = new os::Mutex;
-  commandSig = new os::Condition(commandLock);
-
-  runLock = new os::Mutex;
-
   // Initialise the desktop
   desktop.setStatusLocation(&isDesktopStarted);
   desktop.setQueryConnectionHandler(this);
@@ -95,15 +93,8 @@ VNCServerWin32::~VNCServerWin32() {
   desktop.setStatusLocation(nullptr);
 
   // Join the Accept/Reject dialog thread
-  if (queryConnectDialog) {
-    queryConnectDialog->wait();
+  if (queryConnectDialog)
     delete queryConnectDialog;
-  }
-
-  delete runLock;
-
-  delete commandSig;
-  delete commandLock;
 }
 
 
@@ -123,13 +114,8 @@ void VNCServerWin32::processAddressChange() {
   else
     addrs.push_front("Not accepting connections");
 
-  // Allocate space for the new tip
-  std::list<std::string>::iterator i, next_i;
-  int length = strlen(prefix)+1;
-  for (i=addrs.begin(); i!= addrs.end(); i++)
-    length += i->size() + 1;
-
   // Build the new tip
+  std::list<std::string>::iterator i, next_i;
   std::string toolTip(prefix);
   for (i=addrs.begin(); i!= addrs.end(); i=next_i) {
     next_i = i; next_i ++;
@@ -158,7 +144,7 @@ void VNCServerWin32::regConfigChanged() {
 
 int VNCServerWin32::run() {
   {
-    os::AutoMutex a(runLock);
+    const std::lock_guard<std::mutex> a(runLock);
     thread_id = GetCurrentThreadId();
     runServer = true;
   }
@@ -188,7 +174,7 @@ int VNCServerWin32::run() {
     while (runServer) {
       result = sockMgr.getMessage(&msg, nullptr, 0, 0);
       if (result < 0)
-        throw rdr::win32_error("getMessage", GetLastError());
+        throw core::win32_error("getMessage", GetLastError());
       if (!isServiceProcess() && (result == 0))
         break;
       TranslateMessage(&msg);
@@ -196,7 +182,7 @@ int VNCServerWin32::run() {
     }
 
     vlog.debug("Server exited cleanly");
-  } catch (rdr::win32_error &s) {
+  } catch (core::win32_error &s) {
     vlog.error("%s", s.what());
     result = s.err;
   } catch (std::exception &e) {
@@ -204,7 +190,7 @@ int VNCServerWin32::run() {
   }
 
   {
-    os::AutoMutex a(runLock);
+    const std::lock_guard<std::mutex> a(runLock);
     runServer = false;
     thread_id = (DWORD)-1;
   }
@@ -213,7 +199,7 @@ int VNCServerWin32::run() {
 }
 
 void VNCServerWin32::stop() {
-  os::AutoMutex a(runLock);
+  const std::lock_guard<std::mutex> a(runLock);
   runServer = false;
   if (thread_id != (DWORD)-1)
     PostThreadMessage(thread_id, WM_QUIT, 0, 0);
@@ -270,17 +256,17 @@ void VNCServerWin32::queryConnectionComplete() {
 
 
 bool VNCServerWin32::queueCommand(Command cmd, const void* data, int len, bool wait) {
-  os::AutoMutex a(commandLock);
+  std::unique_lock<std::mutex> lock(commandLock);
   while (command != NoCommand)
-    commandSig->wait();
+    commandSig.wait(lock);
   command = cmd;
   commandData = data;
   commandDataLen = len;
   SetEvent(commandEvent);
   if (wait) {
     while (command != NoCommand)
-      commandSig->wait();
-    commandSig->signal();
+      commandSig.wait(lock);
+    commandSig.notify_one();
   }
   return true;
 }
@@ -291,7 +277,7 @@ void VNCServerWin32::processEvent(HANDLE event_) {
   if (event_ == commandEvent.h) {
     // If there is no command queued then return immediately
     {
-      os::AutoMutex a(commandLock);
+      const std::lock_guard<std::mutex> a(commandLock);
       if (command == NoCommand)
         return;
     }
@@ -321,7 +307,6 @@ void VNCServerWin32::processEvent(HANDLE event_) {
       vncServer.approveConnection(queryConnectDialog->getSock(),
                                   queryConnectDialog->isAccepted(),
                                   "Connection rejected by user");
-      queryConnectDialog->wait();
       delete queryConnectDialog;
       queryConnectDialog = nullptr;
       break;
@@ -332,9 +317,9 @@ void VNCServerWin32::processEvent(HANDLE event_) {
 
     // Clear the command and signal completion
     {
-      os::AutoMutex a(commandLock);
+      std::unique_lock<std::mutex> lock(commandLock);
       command = NoCommand;
-      commandSig->signal();
+      commandSig.notify_one();
     }
   } else if ((event_ == sessionEvent.h) ||
              (event_ == desktop.getTerminateEvent())) {
